"""Take `format-dev --stability-check` output and shrink all stability errors into a
single Python file. Used to update https://github.com/astral-sh/ruff/issues/5828 ."""

from __future__ import annotations

import json
import os
from concurrent.futures import ThreadPoolExecutor, as_completed
from pathlib import Path
from subprocess import check_output
from tempfile import NamedTemporaryFile

from tqdm import tqdm

root = Path(
    check_output(["git", "rev-parse", "--show-toplevel"], text=True).strip(),
)
target = root.joinpath("target")

error_report = target.joinpath("formatter-ecosystem-errors.txt")
error_lines_prefix = "Unstable formatting "


def get_filenames() -> list[str]:
    files = []
    for line in error_report.read_text().splitlines():
        if not line.startswith(error_lines_prefix):
            continue
        files.append(line.removeprefix(error_lines_prefix))
    return files


def shrink_file(file: str) -> tuple[str, str]:
    """Returns filename and minimization"""
    with NamedTemporaryFile(suffix=".py") as temp_file:
        print(f"Starting {file}")
        ruff_dev = target.joinpath("release").joinpath("ruff_dev")
        check_output(
            [
                target.joinpath("release").joinpath("ruff_shrinking"),
                file,
                temp_file.name,
                "Unstable formatting",
                f"{ruff_dev} format-dev --stability-check {temp_file.name}",
            ],
        )
        print(f"Finished {file}")
        return file, Path(temp_file.name).read_text()


def main():
    storage = target.joinpath("minimizations.json")
    output_file = target.joinpath("minimizations.py")
    if storage.is_file():
        outputs = json.loads(storage.read_text())
    else:
        outputs = {}
    files = sorted(set(get_filenames()) - set(outputs))
    # Each process will saturate one core
    with ThreadPoolExecutor(max_workers=os.cpu_count()) as executor:
        tasks = [executor.submit(shrink_file, file) for file in files]
        for future in tqdm(as_completed(tasks), total=len(files)):
            file, output = future.result()
            outputs[file] = output
            storage.write_text(json.dumps(outputs, indent=4))

    # Write to one shareable python file
    with output_file.open("w") as formatted:
        for file, code in sorted(json.loads(storage.read_text()).items()):
            file = file.split("/target/checkouts/")[1]
            formatted.write(f"# {file}\n{code}\n")


if __name__ == "__main__":
    main()



import pytest


@pytest.fixture()
def my_fixture():  # OK no args
    return 0


@pytest.fixture(scope="module")
def my_fixture():  # OK only kwargs
    return 0


@pytest.fixture("module")
def my_fixture():  # Error only args
    return 0


@pytest.fixture("module", autouse=True)
def my_fixture():  # Error mixed
    return 0



# TDO003 - accepted
# TODO: this comment has a link
# https://github.com/astral-sh/ruff/issues/3870

# TODO: this comment has an issue
# TDO-3870

# TDO003 - errors
# TODO: this comment has no
# link after it

# TODO: here's a TODO with no link after it
def foo(x):
    return x

# TODO: here's a TODO on the last line with no link
# Here's more content.
# TDO-3870

# TODO: here's a TODO on the last line with no link
# Here's more content, with a space.

# TDO-3870

# TODO: here's a TODO without an issue link
# TODO: followed by a new TODO with an issue link
# TDO-3870

# foo # TODO: no link!

# TODO: here's a TODO on the last line with no link



"""Test that runtime typing references are properly attributed to scoped imports."""

from __future__ import annotations

from typing import TYPE_CHECKING, cast

if TYPE_CHECKING:
    from threading import Thread


def fn(thread: Thread):
    from threading import Thread

    # The `Thread` on the left-hand side should resolve to the `Thread` imported at the
    # top level.
    x: Thread


def fn(thread: Thread):
    from threading import Thread

    # The `Thread` on the left-hand side should resolve to the `Thread` imported at the
    # top level.
    cast("Thread", thread)


def fn(thread: Thread):
    from threading import Thread

    # The `Thread` on the right-hand side should resolve to the`Thread` imported within
    # `fn`.
    cast(Thread, thread)



import logging
import sys

# G201
try:
    pass
except:
    logging.error("Hello World", exc_info=True)

try:
    pass
except:
    logging.error("Hello World", exc_info=sys.exc_info())

# OK
try:
    pass
except:
    logging.error("Hello World", exc_info=False)

logging.error("Hello World", exc_info=sys.exc_info())



from __future__ import annotations

from datetime import date

from sqlalchemy.orm import DeclarativeBase, Mapped, mapped_column


class Birthday(DeclarativeBase):

    __tablename__ = "birthday"
    id: Mapped[int] = mapped_column(primary_key=True)
    day: Mapped[date]



import os.path
from pathlib import Path
from os.path import getmtime


os.path.getmtime("filename")
os.path.getmtime(b"filename")
os.path.getmtime(Path("filename"))


getmtime("filename")
getmtime(b"filename")
getmtime(Path("filename"))



from paramiko import client
from paramiko.client import AutoAddPolicy, WarningPolicy

ssh_client = client.SSHClient()

# OK
ssh_client.set_missing_host_key_policy(policy=foo)
ssh_client.set_missing_host_key_policy(client.MissingHostKeyPolicy)
ssh_client.set_missing_host_key_policy()
ssh_client.set_missing_host_key_policy(foo)

# Errors
ssh_client.set_missing_host_key_policy(client.AutoAddPolicy)
ssh_client.set_missing_host_key_policy(client.WarningPolicy)
ssh_client.set_missing_host_key_policy(AutoAddPolicy)
ssh_client.set_missing_host_key_policy(policy=client.AutoAddPolicy)
ssh_client.set_missing_host_key_policy(policy=client.WarningPolicy)
ssh_client.set_missing_host_key_policy(policy=WarningPolicy)

# Unrelated
set_missing_host_key_policy(client.AutoAddPolicy)
foo.set_missing_host_key_policy(client.AutoAddPolicy)
ssh_client = 1
ssh_client.set_missing_host_key_policy(client.AutoAddPolicy)



(
    f'{one}'
    f'{two}'
)


rf"Not-so-tricky \"quote"

# Regression test for fstrings dropping comments
result_f = (
    'Traceback (most recent call last):\n'
    f'  File "{__file__}", line {lineno_f+5}, in _check_recursive_traceback_display\n'
    '    f()\n'
    f'  File "{__file__}", line {lineno_f+1}, in f\n'
    '    f()\n'
    f'  File "{__file__}", line {lineno_f+1}, in f\n'
    '    f()\n'
    f'  File "{__file__}", line {lineno_f+1}, in f\n'
    '    f()\n'
    # XXX: The following line changes depending on whether the tests
    # are run through the interactive interpreter or with -m
    # It also varies depending on the platform (stack size)
    # Fortunately, we don't care about exactness here, so we use regex
    r'  \[Previous line repeated (\d+) more times\]' '\n'
    'RecursionError: maximum recursion depth exceeded\n'
)


# Regression for fstring dropping comments that were accidentally attached to
# an expression inside a formatted value
(
    f'{1}'
    # comment
    ''
)

(
    f'{1}'  # comment
    f'{2}'
)

(
    f'{1}'
    f'{2}'  # comment
)

(
    1, (  # comment
        f'{2}'
    )
)

(
    (
        f'{1}'
        # comment
    ),
    2
)



field01: int = 0xFFFFFFFF
field02: int = 0xFFFFFFFFF  # Error: PYI054
field03: int = -0xFFFFFFFF
field04: int = -0xFFFFFFFFF  # Error: PYI054

field05: int = 1234567890
field06: int = 12_456_890
field07: int = 12345678901  # Error: PYI054
field08: int = -1234567801
field09: int = -234_567_890  # Error: PYI054

field10: float = 123.456789
field11: float = 123.4567890  # Error: PYI054
field12: float = -123.456789
field13: float = -123.567_890  # Error: PYI054

field14: complex = 1e1234567j
field15: complex = 1e12345678j  # Error: PYI054
field16: complex = -1e1234567j
field17: complex = 1e123456789j  # Error: PYI054



from pickle import PicklingError, UnpicklingError
import socket

import pytest


def test_ok():
    with pytest.raises(ValueError, match="Can't divide by 0"):
        raise ValueError("Can't divide by 0")


def test_ok_different_error_from_config():
    with pytest.raises(ZeroDivisionError):
        raise ZeroDivisionError("Can't divide by 0")


def test_error_no_argument_given():
    with pytest.raises(ValueError):
        raise ValueError("Can't divide 1 by 0")

    with pytest.raises(socket.error):
        raise ValueError("Can't divide 1 by 0")

    with pytest.raises(PicklingError):
        raise PicklingError("Can't pickle")

    with pytest.raises(UnpicklingError):
        raise UnpicklingError("Can't unpickle")


def test_error_match_is_empty():
    with pytest.raises(ValueError, match=None):
        raise ValueError("Can't divide 1 by 0")

    with pytest.raises(ValueError, match=""):
        raise ValueError("Can't divide 1 by 0")

    with pytest.raises(ValueError, match=f""):
        raise ValueError("Can't divide 1 by 0")



import pytest


@pytest.mark.asyncio()
async def test_something():  # Ok not fixture
    pass


@pytest.mark.asyncio
async def test_something():  # Ok not fixture no parens
    pass


@pytest.mark.asyncio()
@pytest.fixture()
async def my_fixture():  # Error before
    return 0


@pytest.mark.asyncio
@pytest.fixture()
async def my_fixture():  # Error before no parens
    return 0


@pytest.fixture()
@pytest.mark.asyncio()
async def my_fixture():  # Error after
    return 0


@pytest.fixture()
@pytest.mark.asyncio
async def my_fixture():  # Error after no parens
    return 0



import typing
from collections.abc import Mapping
from typing import (
    Annotated,
    TypeAlias,
    Union,
    Literal,
)

just_literals_pipe_union: TypeAlias = (
    Literal[True] | Literal["idk"]
)  # PYI042, since not camel case
PublicAliasT: TypeAlias = str | int
PublicAliasT2: TypeAlias = Union[str, bytes]
_ABCDEFGHIJKLMNOPQRST: TypeAlias = typing.Any
_PrivateAliasS: TypeAlias = Literal["I", "guess", "this", "is", "okay"]
_PrivateAliasS2: TypeAlias = Annotated[str, "also okay"]

snake_case_alias1: TypeAlias = str | int  # PYI042, since not camel case
_snake_case_alias2: TypeAlias = Literal["whatever"]  # PYI042, since not camel case
Snake_case_alias: TypeAlias = int | float  # PYI042, since not camel case

# check that this edge case doesn't crash
_: TypeAlias = str | int



foo = 1
bar = 2
baz = 3

# Errors.
foo = foo
bar = bar
foo, bar = foo, bar
bar, foo = bar, foo
(foo, bar) = (foo, bar)
(bar, foo) = (bar, foo)
foo, (bar, baz) = foo, (bar, baz)
bar, (foo, baz) = bar, (foo, baz)
(foo, bar), baz = (foo, bar), baz
(foo, (bar, baz)) = (foo, (bar, baz))
foo, bar = foo, 1
bar, foo = bar, 1
(foo, bar) = (foo, 1)
(bar, foo) = (bar, 1)
foo, (bar, baz) = foo, (bar, 1)
bar, (foo, baz) = bar, (foo, 1)
(foo, bar), baz = (foo, bar), 1
(foo, (bar, baz)) = (foo, (bar, 1))
foo: int = foo
bar: int = bar

# Non-errors.
foo = bar
bar = foo
foo, bar = bar, foo
foo, bar = bar, foo
(foo, bar) = (bar, foo)
foo, bar = bar, 1
bar, foo = foo, 1
foo: int = bar
bar: int = 1


class Foo:
    foo = foo
    bar = bar



def bar():
    ...  # OK


def foo():
    """foo"""  # OK, docstrings are handled by another rule


def buzz():
    print("buzz")  # ERROR PYI010


def foo2():
    123  # ERROR PYI010


def bizz():
    x = 123  # ERROR PYI010


def foo3():
    pass  # OK, pass is handled by another rule



# basic usage

type X = int
type X = int | str
type X = int | "ForwardRefY"
type X[T] = T | list[X[T]]  # recursive
type X[T] = int
type X[T] = list[T] | set[T]
type X[T, *Ts, **P] = (T, Ts, P)
type X[T: int, *Ts, **P] = (T, Ts, P)
type X[T: (int, str), *Ts, **P] = (T, Ts, P)

# long name
type Xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx = int
type Xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx[A] = int
type Xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx[Aaaaaaaaaaaaaaaaaaaaaaaaaaaa] = int
type Xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx[Aaaaaaaaaaaaaaaaaaaaaaaaaaaa, Bbbbbbbbbbbbb] = int
type Xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx = Tttttttttttttttttttttttttttttttttttttttttttttttttttttttt

# long value
type X = Ttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttt
type X = Aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa | Bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb | Ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

# soft keyword as alias name
type type = int
type match = int
type case = int

# soft keyword as value
type foo = type
type foo = match
type foo = case

# multine definitions
type \
	X = int
type X \
	= int
type X = \
	int
type X = (
    int
)
type \
    X[T] = T
type X \
    [T] = T
type X[T] \
    = T

# type leading comment
type X = ( # trailing open paren comment
    # value leading comment
    int # value trailing comment
    # leading close paren comment
) # type trailing comment


# type leading comment
type X = (
    # value leading comment
    int # value trailing comment

    # leading close paren comment
)

# type parameters
type type_params_single_line[aaaaaaaaaaaaaaaaaaaaaaaaaaaaa, bbbbbbbbbbbbbbb, ccccccccccccccccc] = int
type type_params_arguments_on_their_own_line[aaaaaaaaaaaaaaaaaaaaaaaaaaaaa, bbbbbbbbbbbbbbb, ccccccccccc, ddddddddddddd, eeeeeee] = int
type type_params_argument_per_line[aaaaaaaaaaaaaaaaaaaaaaaaaaaaa, bbbbbbbbbbbbbbb, ccccccccccccccccc, ddddddddddddd, eeeeeeeeeeeeeeee, ffffffffffff] = int
type type_params_trailing_comma[a, b,] = int
type type_params_comments[ # trailing open bracket comment
    # leading comment
    A,

    # in between comment

    B,
    # another leading comment
    C,
    D, # trailing comment
    # leading close bracket comment
] = int  # trailing value comment
type type_params_all_kinds[type_var, *type_var_tuple, **param_spec] = int

# type variable bounds
type bounds_single_line[T: (aaaaaaaaaaaaaaaaaaaaaaaaaaaaa, bbbbbbbbbbbbbbb, ccccccccccccccccc)] = T
type bounds_arguments_on_their_own_line[T: (aaaaaaaaaaaaaaaaaaaaaaaaaaaaa, bbbbbbbbbbbbbbb, ccccccccccc, ddddddddddddd, eeeeeee)] = T
type bounds_argument_per_line[T: (aaaaaaaaaaaaaaaaaaaaaaaaaaaaa, bbbbbbbbbbbbbbb, ccccccccccccccccc, ddddddddddddd, eeeeeeeeeeeeeeee, ffffffffffff)] = T
type bounds_trailing_comma[T: (a, b,)] = T



# Errors.
class Foo:
    __slots__ = "bar"

    def __init__(self, bar):
        self.bar = bar


class Foo:
    __slots__: str = "bar"

    def __init__(self, bar):
        self.bar = bar


class Foo:
    __slots__: str = f"bar"

    def __init__(self, bar):
        self.bar = bar


# Non-errors.
class Foo:
    __slots__ = ("bar",)

    def __init__(self, bar):
        self.bar = bar


class Foo:
    __slots__: tuple[str, ...] = ("bar",)

    def __init__(self, bar):
        self.bar = bar



from os import chmod, mkdir, makedirs, rename, replace, rmdir, sep
from os import remove, unlink, getcwd, readlink, stat
from os.path import abspath, exists, expanduser, isdir, isfile, islink
from os.path import isabs, join, basename, dirname, samefile, splitext

p = "/foo"
q = "bar"

a = abspath(p)
aa = chmod(p)
aaa = mkdir(p)
makedirs(p)
rename(p)
replace(p)
rmdir(p)
remove(p)
unlink(p)
getcwd(p)
b = exists(p)
bb = expanduser(p)
bbb = isdir(p)
bbbb = isfile(p)
bbbbb = islink(p)
readlink(p)
stat(p)
isabs(p)
join(p, q)
sep.join((p, q))
sep.join([p, q])
basename(p)
dirname(p)
samefile(p)
splitext(p)
with open(p) as fp:
    fp.read()
open(p).close()



import builtins
from typing import Union

w: builtins.type[int] | builtins.type[str] | builtins.type[complex]
x: type[int] | type[str] | type[float]
y: builtins.type[int] | type[str] | builtins.type[complex]
z: Union[type[float], type[complex]]
z: Union[type[float, int], type[complex]]

def func(arg: type[int] | str | type[float]) -> None: ...

# OK
x: type[int, str, float]
y: builtins.type[int, str, complex]
z: Union[float, complex]

def func(arg: type[int, float] | str) -> None: ...

# OK
item: type[requests_mock.Mocker] | type[httpretty] = requests_mock.Mocker

def func():
    # PYI055
    item: type[requests_mock.Mocker] | type[httpretty] = requests_mock.Mocker



"""
# coding=utf8""" # empty comment

"""
Invalid coding declaration since it is nested inside a docstring
The following empty comment tests for false positives as our implementation visits comments
"""



import collections
from collections import namedtuple
from typing import NamedTuple, TypedDict

lower = 0
CONSTANT = 0
mixedCase = 0
_mixedCase = 0
mixed_Case = 0
myObj1 = collections.namedtuple("MyObj1", ["a", "b"])
myObj2 = namedtuple("MyObj2", ["a", "b"])
Employee = NamedTuple('Employee', [('name', str), ('id', int)])
Point2D = TypedDict('Point2D', {'in': int, 'x-y': int})



import pytest


def test_ok():
    try:
        something()
    except Exception as e:
        something_else()

    with pytest.raises(ZeroDivisionError) as e:
        1 / 0
    assert e.value.message


def test_error():
    try:
        something()
    except Exception as e:
        assert e.message, "blah blah"



from typing import NamedTuple
import typing

# with complex annotations
MyType = NamedTuple("MyType", [("a", int), ("b", tuple[str, ...])])

# with namespace
MyType = typing.NamedTuple("MyType", [("a", int), ("b", str)])

# invalid identifiers (OK)
MyType = NamedTuple("MyType", [("x-y", int), ("b", tuple[str, ...])])

# no fields
MyType = typing.NamedTuple("MyType")

# empty fields
MyType = typing.NamedTuple("MyType", [])

# keywords
MyType = typing.NamedTuple("MyType", a=int, b=tuple[str, ...])

# unfixable
MyType = typing.NamedTuple("MyType", [("a", int)], [("b", str)])
MyType = typing.NamedTuple("MyType", [("a", int)], b=str)
MyType = typing.NamedTuple(typename="MyType", a=int, b=str)



x = [1, 2, 3]
list(list(x))
list(tuple(x))
tuple(list(x))
tuple(tuple(x))
set(set(x))
set(list(x))
set(tuple(x))
set(sorted(x))
set(sorted(x, key=lambda y: y))
set(reversed(x))
sorted(list(x))
sorted(tuple(x))
sorted(sorted(x))
sorted(sorted(x, key=foo, reverse=False), reverse=False, key=foo)
sorted(sorted(x, reverse=True), reverse=True)
sorted(reversed(x))
sorted(list(x), key=lambda y: y)
tuple(
    list(
        [x, 3, "hell"\
        "o"]
    )
)
set(set())
set(list())
set(tuple())
sorted(reversed())

# Nested sorts with differing keyword arguments. Not flagged.
sorted(sorted(x, key=lambda y: y))
sorted(sorted(x, key=lambda y: y), key=lambda x: x)
sorted(sorted(x), reverse=True)
sorted(sorted(x, reverse=False), reverse=True)



import enum
from enum import Enum, unique


class FakeEnum1(enum.Enum):
    A = "A"
    B = "B"
    C = "B"  # PIE796


class FakeEnum2(Enum):
    A = 1
    B = 2
    C = 2  # PIE796


class FakeEnum3(str, Enum):
    A = "1"
    B = "2"
    C = "2"  # PIE796


class FakeEnum4(Enum):
    A = 1.0
    B = 2.5
    C = 2.5  # PIE796


class FakeEnum5(Enum):
    A = 1.0
    B = True
    C = False
    D = False  # PIE796


class FakeEnum6(Enum):
    A = 1
    B = 2
    C = None
    D = None  # PIE796


@enum.unique
class FakeEnum7(enum.Enum):
    A = "A"
    B = "B"
    C = "C"


@unique
class FakeEnum8(Enum):
    A = 1
    B = 2
    C = 2  # PIE796


class FakeEnum9(enum.Enum):
    A = "A"
    B = "B"
    C = "C"


class FakeEnum10(enum.Enum):
    A = enum.auto()
    B = enum.auto()
    C = enum.auto()



import sys
from sys import version_info as python_version

if sys.version_info < (3, 9): ...  # OK

if sys.version_info >= (3, 9): ...  # OK

if sys.version_info == (3, 9): ...  # OK

if sys.version_info <= (3, 10): ...  # OK

if sys.version_info > (3, 10): ...  # OK

if python_version > (3, 10): ...  # OK



# Errors
a = "hello"

# SIM116
if a == "foo":
    return "bar"
elif a == "bar":
    return "baz"
elif a == "boo":
    return "ooh"
else:
    return 42

# SIM116
if a == 1:
    return (1, 2, 3)
elif a == 2:
    return (4, 5, 6)
elif a == 3:
    return (7, 8, 9)
else:
    return (10, 11, 12)

# SIM116
if a == 1:
    return (1, 2, 3)
elif a == 2:
    return (4, 5, 6)
elif a == 3:
    return (7, 8, 9)

# SIM116
if a == "hello 'sir'":
    return (1, 2, 3)
elif a == 'goodbye "mam"':
    return (4, 5, 6)
elif a == """Fairwell 'mister'""":
    return (7, 8, 9)
else:
    return (10, 11, 12)

# SIM116
if a == b"one":
    return 1
elif a == b"two":
    return 2
elif a == b"three":
    return 3

# SIM116
if a == "hello 'sir'":
    return ("hello'", 'hi"', 3)
elif a == 'goodbye "mam"':
    return (4, 5, 6)
elif a == """Fairwell 'mister'""":
    return (7, 8, 9)
else:
    return (10, 11, 12)

# OK
if a == "foo":
    return "bar"
elif a == "bar":
    return baz()
elif a == "boo":
    return "ooh"
else:
    return 42

# OK
if a == b"one":
    return 1
elif b == b"two":
    return 2
elif a == b"three":
    return 3

# SIM116
if func_name == "create":
    return "A"
elif func_name == "modify":
    return "M"
elif func_name == "remove":
    return "D"
elif func_name == "move":
    return "MV"

# OK
def no_return_in_else(platform):
    if platform == "linux":
        return "auditwheel repair -w {dest_dir} {wheel}"
    elif platform == "macos":
        return "delocate-wheel --require-archs {delocate_archs} -w {dest_dir} -v {wheel}"
    elif platform == "windows":
        return ""
    else:
        msg = f"Unknown platform: {platform!r}"
        raise ValueError(msg)



# single-line failures
query1 = "SELECT %s FROM table" % (var,) # bad
query2 = "SELECT var FROM " + table
query3 = "SELECT " + val + " FROM " + table
query4 = "SELECT {} FROM table;".format(var)
query5 = f"SELECT * FROM table WHERE var = {var}"

query6 = "DELETE FROM table WHERE var = %s" % (var,)
query7 = "DELETE FROM table WHERE VAR = " + var
query8 = "DELETE FROM " + table + "WHERE var = " + var
query9 = "DELETE FROM table WHERE var = {}".format(var)
query10 = f"DELETE FROM table WHERE var = {var}"

query11 = "INSERT INTO table VALUES (%s)" % (var,)
query12 = "INSERT INTO TABLE VALUES (" + var + ")"
query13 = "INSERT INTO {} VALUES ({})".format(table, var)
query14 = f"INSERT INTO {table} VALUES var = {var}"

query15 = "UPDATE %s SET var = %s" % (table, var)
query16 = "UPDATE " + table + " SET var = " + var
query17 = "UPDATE {} SET var = {}".format(table, var)
query18 = f"UPDATE {table} SET var = {var}"

query19 = "select %s from table" % (var,)
query20 = "select var from " + table
query21 = "select " + val + " from " + table
query22 = "select {} from table;".format(var)
query23 = f"select * from table where var = {var}"

query24 = "delete from table where var = %s" % (var,)
query25 = "delete from table where var = " + var
query26 = "delete from " + table + "where var = " + var
query27 = "delete from table where var = {}".format(var)
query28 = f"delete from table where var = {var}"

query29 = "insert into table values (%s)" % (var,)
query30 = "insert into table values (" + var + ")"
query31 = "insert into {} values ({})".format(table, var)
query32 = f"insert into {table} values var = {var}"

query33 = "update %s set var = %s" % (table, var)
query34 = "update " + table + " set var = " + var
query35 = "update {} set var = {}".format(table, var)
query36 = f"update {table} set var = {var}"

# multi-line failures
def query37():
    return """
    SELECT *
    FROM table
    WHERE var = %s
    """ % var

def query38():
    return """
    SELECT *
    FROM TABLE
    WHERE var =
    """ + var

def query39():
    return """
    SELECT *
    FROM table
    WHERE var = {}
    """.format(var)

def query40():
    return f"""
    SELECT *
    FROM table
    WHERE var = {var}
    """

def query41():
    return (
        "SELECT * "
        "FROM table "
        f"WHERE var = {var}"
    )

# # cursor-wrapped failures
query42 = cursor.execute("SELECT * FROM table WHERE var = %s" % var)
query43 = cursor.execute(f"SELECT * FROM table WHERE var = {var}")
query44 = cursor.execute("SELECT * FROM table WHERE var = {}".format(var))
query45 = cursor.executemany("SELECT * FROM table WHERE var = %s" % var, [])

# # pass
query = "SELECT * FROM table WHERE id = 1"
query = "DELETE FROM table WHERE id = 1"
query = "INSERT INTO table VALUES (1)"
query = "UPDATE table SET id = 1"
cursor.execute('SELECT * FROM table WHERE id = %s', var)
cursor.execute('SELECT * FROM table WHERE id = 1')
cursor.executemany('SELECT * FROM table WHERE id = %s', [var, var2])

# # INSERT without INTO (e.g. MySQL and derivatives)
query = "INSERT table VALUES (%s)" % (var,)

# # REPLACE (e.g. MySQL and derivatives, SQLite)
query = "REPLACE INTO table VALUES (%s)" % (var,)
query = "REPLACE table VALUES (%s)" % (var,)

query = "Deselect something that is not SQL even though it has a ' from ' somewhere in %s." % "there"



import re
from re import sub

# B034
re.sub("a", "b", "aaa", re.IGNORECASE)
re.sub("a", "b", "aaa", 5)
re.sub("a", "b", "aaa", 5, re.IGNORECASE)
re.subn("a", "b", "aaa", re.IGNORECASE)
re.subn("a", "b", "aaa", 5)
re.subn("a", "b", "aaa", 5, re.IGNORECASE)
re.split(" ", "a a a a", re.I)
re.split(" ", "a a a a", 2)
re.split(" ", "a a a a", 2, re.I)
sub("a", "b", "aaa", re.IGNORECASE)

# OK
re.sub("a", "b", "aaa")
re.sub("a", "b", "aaa", flags=re.IGNORECASE)
re.sub("a", "b", "aaa", count=5)
re.sub("a", "b", "aaa", count=5, flags=re.IGNORECASE)
re.subn("a", "b", "aaa")
re.subn("a", "b", "aaa", flags=re.IGNORECASE)
re.subn("a", "b", "aaa", count=5)
re.subn("a", "b", "aaa", count=5, flags=re.IGNORECASE)
re.split(" ", "a a a a", flags=re.I)
re.split(" ", "a a a a", maxsplit=2)
re.split(" ", "a a a a", maxsplit=2, flags=re.I)



import typing
from typing import Annotated, Any, Literal, Optional, Tuple, Union


def f(arg: int):
    pass


def f(arg=None):
    pass


def f(arg: Any = None):
    pass


def f(arg: object = None):
    pass


def f(arg: int = None):  # RUF013
    pass


def f(arg: str = None):  # RUF013
    pass


def f(arg: typing.List[str] = None):  # RUF013
    pass


def f(arg: Tuple[str] = None):  # RUF013
    pass


# Optional


def f(arg: Optional[int] = None):
    pass


def f(arg: typing.Optional[int] = None):
    pass


# Union


def f(arg: Union[None] = None):
    pass


def f(arg: Union[None, int] = None):
    pass


def f(arg: Union[str, None] = None):
    pass


def f(arg: typing.Union[int, str, None] = None):
    pass


def f(arg: Union[int, str, Any] = None):
    pass


def f(arg: Union = None):  # RUF013
    pass


def f(arg: Union[int] = None):  # RUF013
    pass


def f(arg: Union[int, str] = None):  # RUF013
    pass


def f(arg: typing.Union[int, str] = None):  # RUF013
    pass


# PEP 604 Union


def f(arg: None | int = None):
    pass


def f(arg: int | None = None):
    pass


def f(arg: int | float | str | None = None):
    pass


def f(arg: int | float = None):  # RUF013
    pass


def f(arg: int | float | str | bytes = None):  # RUF013
    pass


# Literal


def f(arg: None = None):
    pass


def f(arg: Literal[None] = None):
    pass


def f(arg: Literal[1, 2, None, 3] = None):
    pass


def f(arg: Literal[1] = None):  # RUF013
    pass


def f(arg: Literal[1, "foo"] = None):  # RUF013
    pass


def f(arg: typing.Literal[1, "foo", True] = None):  # RUF013
    pass


# Annotated


def f(arg: Annotated[Optional[int], ...] = None):
    pass


def f(arg: Annotated[Union[int, None], ...] = None):
    pass


def f(arg: Annotated[Any, ...] = None):
    pass


def f(arg: Annotated[int, ...] = None):  # RUF013
    pass


def f(arg: Annotated[Annotated[int | str, ...], ...] = None):  # RUF013
    pass


# Multiple arguments


def f(
    arg1: Optional[int] = None,
    arg2: Union[int, None] = None,
    arg3: Literal[1, 2, None, 3] = None,
):
    pass


def f(
    arg1: int = None,  # RUF013
    arg2: Union[int, float] = None,  # RUF013
    arg3: Literal[1, 2, 3] = None,  # RUF013
):
    pass


# Nested


def f(arg: Literal[1, "foo", Literal[True, None]] = None):
    pass


def f(arg: Union[int, Union[float, Union[str, None]]] = None):
    pass


def f(arg: Union[int, Union[float, Optional[str]]] = None):
    pass


def f(arg: Union[int, Literal[True, None]] = None):
    pass


def f(arg: Union[Annotated[int, ...], Annotated[Optional[float], ...]] = None):
    pass


def f(arg: Union[Annotated[int, ...], Union[str, bytes]] = None):  # RUF013
    pass


# Quoted


def f(arg: "int" = None):  # RUF013
    pass


def f(arg: "str" = None):  # RUF013
    pass


def f(arg: "st" "r" = None):  # RUF013
    pass


def f(arg: "Optional[int]" = None):
    pass


def f(arg: Union["int", "str"] = None):  # RUF013
    pass


def f(arg: Union["int", "None"] = None):
    pass


def f(arg: Union["No" "ne", "int"] = None):
    pass


# Avoid flagging when there's a parse error in the forward reference
def f(arg: Union["<>", "int"] = None):
    pass


# Type aliases

Text = str | bytes


def f(arg: Text = None):
    pass


def f(arg: "Text" = None):
    pass


from custom_typing import MaybeInt


def f(arg: MaybeInt = None):
    pass



import asyncio


# Error
def f():
    asyncio.create_task(coordinator.ws_connect())  # Error


# Error
def f():
    asyncio.ensure_future(coordinator.ws_connect())  # Error


# OK
def f():
    background_tasks = set()

    for i in range(10):
        task = asyncio.create_task(some_coro(param=i))

        # Add task to the set. This creates a strong reference.
        background_tasks.add(task)

        # To prevent keeping references to finished tasks forever,
        # make each task remove its own reference from the set after
        # completion:
        task.add_done_callback(background_tasks.discard)


# OK
def f():
    background_tasks = set()

    for i in range(10):
        task = asyncio.ensure_future(some_coro(param=i))

        # Add task to the set. This creates a strong reference.
        background_tasks.add(task)

        # To prevent keeping references to finished tasks forever,
        # make each task remove its own reference from the set after
        # completion:
        task.add_done_callback(background_tasks.discard)


# OK
def f():
    ctx.task = asyncio.create_task(make_request())


# OK
def f():
    tasks.append(asyncio.create_task(self._populate_collection(coll, coll_info)))


# OK
def f():
    asyncio.wait([asyncio.create_task(client.close()) for client in clients.values()])


# OK
def f():
    tasks = [asyncio.create_task(task) for task in tasks]


# OK (false negative)
def f():
    task = asyncio.create_task(coordinator.ws_connect())


# OK (potential false negative)
def f():
    do_nothing_with_the_task(asyncio.create_task(coordinator.ws_connect()))


# Error
def f():
    loop = asyncio.get_running_loop()
    loop.create_task(coordinator.ws_connect())  # Error


# OK
def f():
    loop.create_task(coordinator.ws_connect())


# OK
def f():
    loop = asyncio.get_running_loop()
    loop.do_thing(coordinator.ws_connect())



x = range(10)

# RUF015
list(x)[0]
tuple(x)[0]
list(i for i in x)[0]
[i for i in x][0]

# OK (not indexing (solely) the first element)
list(x)
list(x)[1]
list(x)[-1]
list(x)[1:]
list(x)[:3:2]
list(x)[::2]
list(x)[::]
[i for i in x]
[i for i in x][1]
[i for i in x][-1]
[i for i in x][:1]
[i for i in x][:1:1]
[i for i in x][:1:2]
[i for i in x][1:]
[i for i in x][:3:2]
[i for i in x][::2]
[i for i in x][::]

# RUF015 (doesn't mirror the underlying list)
[i + 1 for i in x][0]
[i for i in x if i > 5][0]
[(i, i + 1) for i in x][0]

# RUF015 (multiple generators)
y = range(10)
[i + j for i in x for j in y][0]

# RUF015
list(range(10))[0]
list(x.y)[0]
list(x["y"])[0]

# RUF015 (multi-line)
revision_heads_map_ast = [
    a
    for a in revision_heads_map_ast_obj.body
    if isinstance(a, ast.Assign) and a.targets[0].id == "REVISION_HEADS_MAP"
][0]



"""
Violation:

Prefer TypeError when relevant.
"""


def incorrect_basic(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        raise Exception("...")  # should be typeerror


def incorrect_multiple_type_check(some_arg):
    if isinstance(some_arg, (int, str)):
        pass
    else:
        raise Exception("...")  # should be typeerror


class MyClass:
    pass


def incorrect_with_issubclass(some_arg):
    if issubclass(some_arg, MyClass):
        pass
    else:
        raise Exception("...")  # should be typeerror


def incorrect_with_callable(some_arg):
    if callable(some_arg):
        pass
    else:
        raise Exception("...")  # should be typeerror


def incorrect_ArithmeticError(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        raise ArithmeticError("...")  # should be typeerror


def incorrect_AssertionError(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        raise AssertionError("...")  # should be typeerror


def incorrect_AttributeError(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        raise AttributeError("...")  # should be typeerror


def incorrect_BufferError(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        raise BufferError  # should be typeerror


def incorrect_EOFError(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        raise EOFError("...")  # should be typeerror


def incorrect_ImportError(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        raise ImportError("...")  # should be typeerror


def incorrect_LookupError(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        raise LookupError("...")  # should be typeerror


def incorrect_MemoryError(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        # should be typeerror
        # not multiline is on purpose for fix
        raise MemoryError(
            "..."
        )


def incorrect_NameError(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        raise NameError("...")  # should be typeerror


def incorrect_ReferenceError(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        raise ReferenceError("...")  # should be typeerror


def incorrect_RuntimeError(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        raise RuntimeError("...")  # should be typeerror


def incorrect_SyntaxError(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        raise SyntaxError("...")  # should be typeerror


def incorrect_SystemError(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        raise SystemError("...")  # should be typeerror


def incorrect_ValueError(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        raise ValueError("...")  # should be typeerror


def incorrect_not_operator_isinstance(some_arg):
    if not isinstance(some_arg):
        pass
    else:
        raise Exception("...")  # should be typeerror


def incorrect_and_operator_isinstance(arg1, arg2):
    if isinstance(some_arg) and isinstance(arg2):
        pass
    else:
        raise Exception("...")  # should be typeerror


def incorrect_or_operator_isinstance(arg1, arg2):
    if isinstance(some_arg) or isinstance(arg2):
        pass
    else:
        raise Exception("...")  # should be typeerror


def incorrect_multiple_operators_isinstance(arg1, arg2, arg3):
    if not isinstance(arg1) and isinstance(arg2) or isinstance(arg3):
        pass
    else:
        raise Exception("...")  # should be typeerror


def incorrect_not_operator_callable(some_arg):
    if not callable(some_arg):
        pass
    else:
        raise Exception("...")  # should be typeerror


def incorrect_and_operator_callable(arg1, arg2):
    if callable(some_arg) and callable(arg2):
        pass
    else:
        raise Exception("...")  # should be typeerror


def incorrect_or_operator_callable(arg1, arg2):
    if callable(some_arg) or callable(arg2):
        pass
    else:
        raise Exception("...")  # should be typeerror


def incorrect_multiple_operators_callable(arg1, arg2, arg3):
    if not callable(arg1) and callable(arg2) or callable(arg3):
        pass
    else:
        raise Exception("...")  # should be typeerror


def incorrect_not_operator_issubclass(some_arg):
    if not issubclass(some_arg):
        pass
    else:
        raise Exception("...")  # should be typeerror


def incorrect_and_operator_issubclass(arg1, arg2):
    if issubclass(some_arg) and issubclass(arg2):
        pass
    else:
        raise Exception("...")  # should be typeerror


def incorrect_or_operator_issubclass(arg1, arg2):
    if issubclass(some_arg) or issubclass(arg2):
        pass
    else:
        raise Exception("...")  # should be typeerror


def incorrect_multiple_operators_issubclass(arg1, arg2, arg3):
    if not issubclass(arg1) and issubclass(arg2) or issubclass(arg3):
        pass
    else:
        raise Exception("...")  # should be typeerror


def incorrect_multi_conditional(arg1, arg2):
    if isinstance(arg1, int):
        pass
    elif isinstance(arg2, int):
        raise Exception("...")  # should be typeerror


def multiple_is_instance_checks(some_arg):
    if isinstance(some_arg, str):
        pass
    elif isinstance(some_arg, int):
        pass
    else:
        raise Exception("...")   # should be typeerror


class MyCustomTypeValidation(Exception):
    pass


def correct_custom_exception(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        raise MyCustomTypeValidation("...")  # that's correct, because it's not vanilla


def correct_complex_conditional(val):
    if val is not None and (not isinstance(val, int) or val < 0):
        raise ValueError(...)  # fine if this is not a TypeError


def correct_multi_conditional(some_arg):
    if some_arg == 3:
        pass
    elif isinstance(some_arg, int):
        pass
    else:
        raise Exception("...")  # fine if this is not a TypeError


def correct_should_ignore(some_arg):
    if isinstance(some_arg, int):
        pass
    else:
        raise TypeError("...")


def check_body(some_args):
    if isinstance(some_args, int):
        raise ValueError("...") # should be typeerror


def check_body_correct(some_args):
    if isinstance(some_args, int):
        raise TypeError("...") # correct


def multiple_elifs(some_args):
    if not isinstance(some_args, int):
        raise ValueError("...") # should be typerror
    elif some_args < 3:
        raise ValueError("...")  # this is ok
    elif some_args > 10:
        raise ValueError("...")  # this is ok if we don't simplify
    else:
        pass


def multiple_ifs(some_args):
    if not isinstance(some_args, int):
        raise ValueError("...") # should be typerror
    else:
        if some_args < 3:
            raise ValueError("...")  # this is ok
        else:
            if some_args > 10:
                raise ValueError("...")  # this is ok if we don't simplify
            else:
                pass


def else_body(obj):
    if isinstance(obj, datetime.timedelta):
        return "TimeDelta"
    elif isinstance(obj, relativedelta.relativedelta):
        return "RelativeDelta"
    elif isinstance(obj, CronExpression):
        return "CronExpression"
    else:
        raise Exception(f"Unknown object type: {obj.__class__.__name__}")


def early_return():
    if isinstance(this, some_type):
        if x in this:
            return

        raise ValueError(f"{this} has a problem")  # this is ok


def early_break():
    for x in this:
        if isinstance(this, some_type):
            if x in this:
                break

            raise ValueError(f"{this} has a problem")  # this is ok


def early_continue():
    for x in this:
        if isinstance(this, some_type):
            if x in this:
                continue

            raise ValueError(f"{this} has a problem")  # this is ok


def early_return_else():
    if isinstance(this, some_type):
        pass
    else:
        if x in this:
            return

        raise ValueError(f"{this} has a problem")  # this is ok



x = (
    'This'
    'is'
    'not'
)

x = (
    'This' \
    'is' \
    'not'
)

x = (
    'This'
    'is "actually"'
    'fine'
)

x = (
    'This' \
    'is "actually"' \
    'fine'
)

if True:
    'This can use "single" quotes'
'But this needs to be changed'



import typing
from typing import TypeAlias

# UP040
x: typing.TypeAlias = int
x: TypeAlias = int

# UP040 simple generic
T = typing.TypeVar["T"]
x: typing.TypeAlias = list[T]

# UP040 call style generic
T = typing.TypeVar("T")
x: typing.TypeAlias = list[T]

# UP040 bounded generic
T = typing.TypeVar("T", bound=int)
x: typing.TypeAlias = list[T]

# UP040 constrained generic
T = typing.TypeVar("T", int, str)
x: typing.TypeAlias = list[T]

# UP040 contravariant generic
T = typing.TypeVar("T", contravariant=True)
x: typing.TypeAlias = list[T]

# UP040 covariant generic
T = typing.TypeVar("T", covariant=True)
x: typing.TypeAlias = list[T]

# UP040 in class scope
T = typing.TypeVar["T"]
class Foo:
    # reference to global variable
    x: typing.TypeAlias = list[T]

    # reference to class variable
    TCLS = typing.TypeVar["TCLS"]
    y: typing.TypeAlias = list[TCLS]

# UP040 wont add generics in fix
T = typing.TypeVar(*args)
x: typing.TypeAlias = list[T]

# OK
x: TypeAlias
x: int = 1



import os
import subprocess
import time


async def foo():
    open("foo")


async def foo():
    time.sleep(1)


async def foo():
    subprocess.run("foo")


async def foo():
    subprocess.call("foo")


async def foo():
    subprocess.foo(0)


async def foo():
    os.wait4(10)


async def foo():
    os.wait(12)



retriable_exceptions = (FileExistsError, FileNotFoundError)

try:
    pass
except (ValueError,):
    pass
except AttributeError:
    pass
except (ImportError, TypeError):
    pass
except (*retriable_exceptions,):
    pass
except(ValueError,):
    pass



for i in range(10):
    print(i)

print(i)  # name no longer defined on Python 3; no warning yet

for i in range(10):  # name not used within the loop; B007
    print(10)

print(i)  # name no longer defined on Python 3; no warning yet


for _ in range(10):  # _ is okay for a throw-away variable
    print(10)


for i in range(10):
    for j in range(10):
        for k in range(10):  # k not used, i and j used transitively
            print(i + j)


def strange_generator():
    for i in range(10):
        for j in range(10):
            for k in range(10):
                for l in range(10):
                    yield i, (j, (k, l))


for i, (j, (k, l)) in strange_generator():  # i, k not used
    print(j, l)

FMT = "{foo} {bar}"
for foo, bar in [(1, 2)]:
    if foo:
        print(FMT.format(**locals()))

for foo, bar in [(1, 2)]:
    if foo:
        print(FMT.format(**globals()))

for foo, bar in [(1, 2)]:
    if foo:
        print(FMT.format(**vars()))

for foo, bar in [(1, 2)]:
    print(FMT.format(foo=foo, bar=eval("bar")))


def f():
    # Fixable.
    for foo, bar, baz in (["1", "2", "3"],):
        if foo or baz:
            break


def f():
    # Unfixable due to usage of `bar` outside of loop.
    for foo, bar, baz in (["1", "2", "3"],):
        if foo or baz:
            break

    print(bar)


def f():
    # Fixable.
    for foo, bar, baz in (["1", "2", "3"],):
        if foo or baz:
            break

    bar = 1


def f():
    # Unfixable.
    for foo, bar, baz in (["1", "2", "3"],):
        if foo or baz:
            break
    else:
        bar = 1

    print(bar)


def f():
    # Unfixable (false negative) due to usage of `bar` outside of loop.
    for foo, bar, baz in (["1", "2", "3"],):
        if foo or baz:
            break

    bar = 1
    print(bar)


# Unfixable due to trailing underscore (`_line_` wouldn't be considered an ignorable
# variable name).
for line_ in range(self.header_lines):
    fp.readline()

# Regression test: visitor didn't walk the elif test
for key, value in current_crawler_tags.items():
    if key:
        pass
    elif wanted_tag_value != value:
        pass



"""Test: imports within `ModuleNotFoundError` and `ImportError` handlers."""


def module_not_found_error():
    try:
        import orjson

        return True
    except ModuleNotFoundError:
        return False


def import_error():
    try:
        import orjson

        return True
    except ImportError:
        return False



import collections
from collections import namedtuple
from typing import NamedTuple, TypedDict


class C:
    lower = 0
    CONSTANT = 0
    mixedCase = 0
    _mixedCase = 0
    mixed_Case = 0
    myObj1 = collections.namedtuple("MyObj1", ["a", "b"])
    myObj2 = namedtuple("MyObj2", ["a", "b"])
    Employee = NamedTuple('Employee', [('name', str), ('id', int)])
    Point2D = TypedDict('Point2D', {'in': int, 'x-y': int})


class D(TypedDict):
    lower: int
    CONSTANT: str
    mixedCase: bool
    _mixedCase: list
    mixed_Case: set



if not aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb:
    pass

a = True
not a

b = 10
-b
+b

## Leading operand comments

if not (
    # comment
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
):
    pass


if ~(
    # comment
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb):
    pass

if -(
    # comment
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb):
    pass


if +(
    # comment
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb):
    pass

if (
    not
    # comment
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
):
    pass


if (
    ~
    # comment
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb):
    pass

if (
    -
    # comment
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb):
    pass


if (
    +
    # comment
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb):
    pass

## Parentheses

if (
    # unary comment
    not
    # operand comment
    (
        # comment
        aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
        + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
    )
):
    pass

if (not (
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
    + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
)
):
    pass

if aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa & (not (
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
    + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
)
):
    pass

if (
    not (
            aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
            + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
        )
        & aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
):
    pass


## Trailing operator comments

if (
    not # comment
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
    + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
):
    pass


if (
    ~ # comment
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
    + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
):
    pass

if (
    - # comment
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
    + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
):
    pass


if (
    + # comment
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
    + bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
):
    pass


## Varia

if not \
    a:
    pass

# Regression: https://github.com/astral-sh/ruff/issues/5338
if a and not aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa & aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa:
    ...

if (
  not
  # comment
  a):
    ...

if (
  not  # comment
  a):
    ...

# Regression test for: https://github.com/astral-sh/ruff/issues/7423
if True:
    if True:
        if True:
            if not yn_question(
                Fore.RED
                + "WARNING: Removing listed files. Do you really want to continue. yes/n)? "
            ):
                pass



# SIM108
if a:
    b = c
else:
    b = d

# OK
b = c if a else d

# OK
if a:
    b = c
elif c:
    b = a
else:
    b = d

# OK
if True:
    pass
elif a:
    b = 1
else:
    b = 2

# SIM108
if True:
    pass
else:
    if a:
        b = 1
    else:
        b = 2


import sys

# OK
if sys.version_info >= (3, 9):
    randbytes = random.randbytes
else:
    randbytes = _get_random_bytes

# OK
if sys.platform == "darwin":
    randbytes = random.randbytes
else:
    randbytes = _get_random_bytes

# OK
if sys.platform.startswith("linux"):
    randbytes = random.randbytes
else:
    randbytes = _get_random_bytes


# SIM108 (without fix due to comments)
if x > 0:
    # test test
    abc = x
else:
    # test test test
    abc = -x


# OK (too long)
if parser.errno == BAD_FIRST_LINE:
    req = wrappers.Request(sock, server=self._server)
else:
    req = wrappers.Request(
        sock,
        parser.get_method(),
        parser.get_scheme() or _scheme,
        parser.get_path(),
        parser.get_version(),
        parser.get_query_string(),
        server=self._server,
    )


# SIM108
if a:
    b = "cccccccccccccccccccccccccccccccccß"
else:
    b = "ddddddddddddddddddddddddddddddddd💣"


# OK (too long)
if True:
    if a:
        b = ccccccccccccccccccccccccccccccccccc
    else:
        b = ddddddddddddddddddddddddddddddddddd


# OK (too long with tabs)
if True:
	if a:
		b = ccccccccccccccccccccccccccccccccccc
	else:
		b = ddddddddddddddddddddddddddddddddddd


# SIM108 (without fix due to trailing comment)
if True:
    exitcode = 0
else:
    exitcode = 1  # Trailing comment


# SIM108
if True: x = 3  # Foo
else: x = 5


# SIM108
if True:  # Foo
    x = 3
else:
    x = 5


# OK
def f():
    if True:
        x = yield 3
    else:
        x = yield 5



import unittest


class Test(unittest.TestCase):
    def test_errors(self):
        with self.assertRaises(ValueError):
            raise ValueError
        with self.assertRaises(expected_exception=ValueError):
            raise ValueError

        with self.failUnlessRaises(ValueError):
            raise ValueError

        with self.assertRaisesRegex(ValueError, "test"):
            raise ValueError("test")

        with self.assertRaisesRegex(ValueError, expected_regex="test"):
            raise ValueError("test")

        with self.assertRaisesRegex(
            expected_exception=ValueError, expected_regex="test"
        ):
            raise ValueError("test")

        with self.assertRaisesRegex(
            expected_regex="test", expected_exception=ValueError
        ):
            raise ValueError("test")

        with self.assertRaisesRegexp(ValueError, "test"):
            raise ValueError("test")

    def test_unfixable_errors(self):
        with self.assertRaises(ValueError, msg="msg"):
            raise ValueError

        with self.assertRaises(
            # comment
            ValueError
        ):
            raise ValueError

        with (
            self
            # comment
            .assertRaises(ValueError)
        ):
            raise ValueError



import os

# Bad
os.environ['foo']

os.environ.get('foo')

os.environ.get('foo', 'bar')

os.getenv('foo')

env = os.environ.get('foo')

env = os.environ['foo']

if env := os.environ.get('foo'):
    pass

if env := os.environ['foo']:
    pass


# Good
os.environ['FOO']

os.environ.get('FOO')

os.environ.get('FOO', 'bar')

os.getenv('FOO')

env = os.getenv('FOO')

if env := os.getenv('FOO'):
    pass

env = os.environ['FOO']

if env := os.environ['FOO']:
    pass



key in obj.keys()  # SIM118

key not in obj.keys()  # SIM118

foo["bar"] in obj.keys()  # SIM118

foo["bar"] not in obj.keys()  # SIM118

foo['bar'] in obj.keys()  # SIM118

foo['bar'] not in obj.keys()  # SIM118

foo() in obj.keys()  # SIM118

foo() not in obj.keys()  # SIM118

for key in obj.keys():  # SIM118
    pass

for key in list(obj.keys()):
    if some_property(key):
        del obj[key]

[k for k in obj.keys()]  # SIM118

{k for k in obj.keys()}  # SIM118

{k: k for k in obj.keys()}  # SIM118

(k for k in obj.keys())  # SIM118

key in (obj or {}).keys()  # SIM118

(key) in (obj or {}).keys()  # SIM118

from typing import KeysView


class Foo:
    def keys(self) -> KeysView[object]:
        ...

    def __contains__(self, key: object) -> bool:
        return key in self.keys()  # OK


# Regression test for: https://github.com/astral-sh/ruff/issues/7124
key in obj.keys()and foo
(key in obj.keys())and foo
key in (obj.keys())and foo

# Regression test for: https://github.com/astral-sh/ruff/issues/7200
for key in (
    self.experiment.surveys[0]
        .stations[0]
        .keys()
):
    continue



import typing
from typing import TypeAlias, Literal, Any

NewAny = Any
OptionalStr = typing.Optional[str]
Foo = Literal["foo"]
IntOrStr = int | str
AliasNone = None

NewAny: typing.TypeAlias = Any
OptionalStr: TypeAlias = typing.Optional[str]
Foo: typing.TypeAlias = Literal["foo"]
IntOrStr: TypeAlias = int | str
IntOrFloat: Foo = int | float
AliasNone: typing.TypeAlias = None

# these are ok
VarAlias = str
AliasFoo = Foo



def func():
    import numpy as np

    np.round_(np.random.rand(5, 5), 2)
    np.product(np.random.rand(5, 5))
    np.cumproduct(np.random.rand(5, 5))
    np.sometrue(np.random.rand(5, 5))
    np.alltrue(np.random.rand(5, 5))


def func():
    from numpy import round_, product, cumproduct, sometrue, alltrue

    round_(np.random.rand(5, 5), 2)
    product(np.random.rand(5, 5))
    cumproduct(np.random.rand(5, 5))
    sometrue(np.random.rand(5, 5))
    alltrue(np.random.rand(5, 5))



import unittest

def badAllowed():
    pass

def stillBad():
    pass

class Test(unittest.TestCase):
    def badAllowed(self):
        return super().tearDown()

    def stillBad(self):
        return super().tearDown()



# T004 - accepted
# TODO(evanrittenhouse): this has a colon
# T004 - errors
# TODO this has no colon
# TODO(evanrittenhouse 😀) this has no colon
# FIXME add a colon
# foo # TODO add a colon
# TODO this has a colon but it doesn't terminate the tag, so this should throw. https://www.google.com



# T001 - accepted
# TODO (evanrittenhouse): this is a valid TODO
# SOME_OTHER_TAG: this is impossible to determine
# this is not a TODO

# T001 - errors
# XXX (evanrittenhouse): this is not fine
# FIXME (evanrittenhouse): this is not fine
# foo # XXX: this isn't fine either



# OK
def f():
    for x in z:
        yield


def f():
    for x in z:
        yield y


def f():
    for x, y in z:
        yield x


def f():
    for x, y in z:
        yield y


def f():
    for a, b in z:
        yield x, y


def f():
    for x, y in z:
        yield y, x


def f():
    for x, y, c in z:
        yield x, y


def f():
    for x in z:
        x = 22
        yield x


def f():
    for x in z:
        yield x
    else:
        print("boom!")


def f():
    for x in range(5):
        yield x
    print(x)


def f():
    def g():
        print(x)

    for x in range(5):
        yield x
    g()


def f():
    def g():
        def h():
            print(x)

        return h

    for x in range(5):
        yield x
    g()()


def f(x):
    for x in y:
        yield x
    del x


async def f():
    for x in y:
        yield x


def f():
    x = 1
    print(x)
    for x in y:
        yield x


def f():
    for x in y:
        yield x
    print(x)


def f():
    for x in y:
        yield x
    z = lambda: x


def f():
    for x in y:
        yield x

    class C:
        def __init__(self):
            print(x)


def f():
    for x in y:
        yield x, x + 1


def f():
    for x, y in z:
        yield x, y, x + y



import typing
from typing import Protocol


class _Foo(Protocol):
    bar: int


class _Bar(typing.Protocol):
    bar: int


# OK
class _UsedPrivateProtocol(Protocol):
    bar: int


def uses__UsedPrivateProtocol(arg: _UsedPrivateProtocol) -> None: ...



from typing import Any
import typing


class Bad:
    def __eq__(self, other: Any) -> bool: ...  # Y032
    def __ne__(self, other: typing.Any) -> typing.Any: ...  # Y032


class Good:
  def __eq__(self, other: object) -> bool: ...

  def __ne__(self, obj: object) -> int: ...


class WeirdButFine:
    def __eq__(self, other: Any, strange_extra_arg: list[str]) -> Any: ...
    def __ne__(self, *, kw_only_other: Any) -> bool: ...


class Unannotated:
  def __eq__(self) -> Any: ...
  def __ne__(self) -> bool: ...




import os.path
from pathlib import Path
from os.path import getsize


os.path.getsize("filename")
os.path.getsize(b"filename")
os.path.getsize(Path("filename"))
os.path.getsize(__file__)

getsize("filename")
getsize(b"filename")
getsize(Path("filename"))
getsize(__file__)



some_dict = {"a": 12, "b": 32, "c": 44}


def f():
    for _, value in some_dict.items():  # PERF102
        print(value)


def f():
    for key, _ in some_dict.items():  # PERF102
        print(key)


def f():
    for weird_arg_name, _ in some_dict.items():  # PERF102
        print(weird_arg_name)


def f():
    for name, (_, _) in some_dict.items():  # PERF102
        print(name)


def f():
    for name, (value1, _) in some_dict.items():  # OK
        print(name, value1)


def f():
    for (key1, _), (_, _) in some_dict.items():  # PERF102
        print(key1)


def f():
    for (_, (_, _)), (value, _) in some_dict.items():  # PERF102
        print(value)


def f():
    for (_, key2), (value1, _) in some_dict.items():  # OK
        print(key2, value1)


def f():
    for ((_, key2), (value1, _)) in some_dict.items():  # OK
        print(key2, value1)


def f():
    for ((_, key2), (_, _)) in some_dict.items():  # PERF102
        print(key2)


def f():
    for (_, _, _, variants), (r_language, _, _, _) in some_dict.items():  # OK
        print(variants, r_language)


def f():
    for (_, _, (_, variants)), (_, (_, (r_language, _))) in some_dict.items():  # OK
        print(variants, r_language)


def f():
    for key, value in some_dict.items():  # OK
        print(key, value)


def f():
    for _, value in some_dict.items(12):  # OK
        print(value)


def f():
    for key in some_dict.keys():  # OK
        print(key)


def f():
    for value in some_dict.values():  # OK
        print(value)


def f():
    for name, (_, _) in (some_function()).items():  # PERF102
        print(name)


def f():
    for name, (_, _) in (some_function().some_attribute).items():  # PERF102
        print(name)


def f():
    for name, unused_value in some_dict.items():  # PERF102
        print(name)


def f():
    for unused_name, value in some_dict.items():  # PERF102
        print(value)


# Regression test for: https://github.com/astral-sh/ruff/issues/7097
def _create_context(name_to_value):
    for(B,D)in A.items():
        if(C:=name_to_value.get(B.name)):A.run(B.set,C)



# Violations of PYI013

class OneAttributeClass:
    value: int
    ...  # Error

class OneAttributeClass2:
    ...  # Error
    value: int

class MyClass:
    ...
    value: int

class TwoEllipsesClass:
    ...
    ...  # Error

class DocstringClass:
    """
    My body only contains an ellipsis.
    """

    ...  # Error

class NonEmptyChild(Exception):
    value: int
    ...  # Error

class NonEmptyChild2(Exception):
    ...  # Error
    value: int

class NonEmptyWithInit:
    value: int
    ...  # Error

    def __init__():
        pass

# Not violations

class EmptyClass: ...
class EmptyEllipsis: ...

class Dog:
    eyes: int = 2

class WithInit:
    value: int = 0

    def __init__(): ...

def function(): ...

...



# error
obj.startswith("foo") or obj.startswith("bar")
# error
obj.endswith("foo") or obj.endswith("bar")
# error
obj.startswith(foo) or obj.startswith(bar)
# error
obj.startswith(foo) or obj.startswith("foo")
# error
obj.endswith(foo) or obj.startswith(foo) or obj.startswith("foo")

# ok
obj.startswith(("foo",  "bar"))
# ok
obj.endswith(("foo",  "bar"))
# ok
obj.startswith("foo") or obj.endswith("bar")
# ok
obj.startswith("foo") or abc.startswith("bar")



from typing import Set
from some.package.name import foo, bar


s = set()
s2 = {}
s3: set[int] = foo()

# these should match

# FURB132
if "x" in s:
    s.remove("x")


# FURB132
if "x" in s2:
    s2.remove("x")


# FURB132
if "x" in s3:
    s3.remove("x")


var = "y"
# FURB132
if var in s:
    s.remove(var)


if f"{var}:{var}" in s:
    s.remove(f"{var}:{var}")


def identity(x):
    return x


# TODO: FURB132
if identity("x") in s2:
    s2.remove(identity("x"))


# these should not

if "x" in s:
    s.remove("y")

s.discard("x")

s2 = set()

if "x" in s:
    s2.remove("x")

if "x" in s:
    s.remove("x")
    print("removed item")

if bar() in s:
    # bar() might have a side effect
    s.remove(bar())

if "x" in s:
    s.remove("x")
else:
    print("not found")

class Container:
    def remove(self, item) -> None:
        return

    def __contains__(self, other) -> bool:
        return True

c = Container()

if "x" in c:
    c.remove("x")



from typing import Dict, List

names = {"key": "value"}
nums = [1, 2, 3]
x = 42
y = "hello"

# these should match

# FURB131
del nums[:]


# FURB131
del names[:]


# FURB131
del x, nums[:]


# FURB131
del y, names[:], x


def yes_one(x: list[int]):
    # FURB131
    del x[:]


def yes_two(x: dict[int, str]):
    # FURB131
    del x[:]


def yes_three(x: List[int]):
    # FURB131
    del x[:]


def yes_four(x: Dict[int, str]):
    # FURB131
    del x[:]


# these should not

del names["key"]
del nums[0]

x = 1
del x


del nums[1:2]


del nums[:2]
del nums[1:]
del nums[::2]


def no_one(param):
    del param[:]



"""
Violation:

Checks for uses of `raise` directly after a `rescue`.
"""
class MyException(Exception):
    pass

def bad():
    try:
        process()
    except Exception:
        raise

def bad():
    try:
        process()
    except Exception:
        raise
        print("this code is pointless!")

def bad():
    try:
        process()
    except:
        # I am a comment, not a statement!
        raise

def bad():
    try:
        process()
    except Exception:
        raise

def bad():
    try:
        process()
    except Exception as e:
        raise

def bad():
    try:
        process()
    except Exception as e:
        raise e

def bad():
    try:
        process()
    except MyException:
        raise
    except Exception:
        raise

def bad():
    try:
        process()
    except MyException as e:
        raise e
    except Exception as e:
        raise e

def bad():
    try:
        process()
    except MyException as ex:
        raise ex
    except Exception as e:
        raise e

def fine():
    try:
        process()
    except Exception as e:
        raise e from None

def fine():
    try:
        process()
    except Exception as e:
        raise e from Exception

def fine():
    try:
        process()
    except Exception as e:
        raise ex

def fine():
    try:
        process()
    except MyException:
        raise
    except Exception:
        print("bar")

def fine():
    try:
        process()
    except Exception:
        print("initiating rapid unscheduled disassembly of program")

def fine():
    try:
        process()
    except MyException:
        print("oh no!")
        raise

def fine():
    try:
        process()
    except Exception:
        if True:
            raise

def fine():
    try:
        process()
    finally:
        # I am a comment, not a statement!
        print("but i am a statement")
        raise



def zipped():
    return zip([1, 2, 3], "ABC")

# Errors.

# FURB140
[print(x, y) for x, y in zipped()]

# FURB140
(print(x, y) for x, y in zipped())

# FURB140
{print(x, y) for x, y in zipped()}


from itertools import starmap as sm

# FURB140
[print(x, y) for x, y in zipped()]

# FURB140
(print(x, y) for x, y in zipped())

# FURB140
{print(x, y) for x, y in zipped()}

# Non-errors.

[print(x, int) for x, _ in zipped()]

[print(x, *y) for x, y in zipped()]

[print(x, y, 1) for x, y in zipped()]

[print(y, x) for x, y in zipped()]

[print(x + 1, y) for x, y in zipped()]

[print(x) for x in range(100)]

[print() for x, y in zipped()]

[print(x, end=y) for x, y in zipped()]



#: E251 E251
def foo(bar = False):
    '''Test function with an error in declaration'''
    pass
#: E251
foo(bar= True)
#: E251
foo(bar =True)
#: E251 E251
foo(bar = True)
#: E251
y = bar(root= "sdasd")
#: E251:2:29
parser.add_argument('--long-option',
                    default=
                    "/rather/long/filesystem/path/here/blah/blah/blah")
#: E251:1:45
parser.add_argument('--long-option', default
                    ="/rather/long/filesystem/path/here/blah/blah/blah")
#: E251:3:8 E251:3:10
foo(True,
    baz=(1, 2),
    biz = 'foo'
    )
#: Okay
foo(bar=(1 == 1))
foo(bar=(1 != 1))
foo(bar=(1 >= 1))
foo(bar=(1 <= 1))
(options, args) = parser.parse_args()
d[type(None)] = _deepcopy_atomic

# Annotated Function Definitions
#: Okay
def munge(input: AnyStr, sep: AnyStr = None, limit=1000,
          extra: Union[str, dict] = None) -> AnyStr:
    pass
#: Okay
async def add(a: int = 0, b: int = 0) -> int:
    return a + b
# Previously E251 four times
#: E271:1:6
async  def add(a: int = 0, b: int = 0) -> int:
    return a + b
#: E252:1:15 E252:1:16 E252:1:27 E252:1:36
def add(a: int=0, b: int =0, c: int= 0) -> int:
    return a + b + c
#: Okay
def add(a: int = _default(name='f')):
    return a



from typing import NoReturn, Never
import typing_extensions

def foo(arg): ...
def foo_int(arg: int): ...
def foo_no_return(arg: NoReturn): ...  # Error: PYI050
def foo_no_return_typing_extensions(
    arg: typing_extensions.NoReturn,
): ...  # Error: PYI050
def foo_no_return_kwarg(arg: int, *, arg2: NoReturn): ...  # Error: PYI050
def foo_no_return_pos_only(arg: int, /, arg2: NoReturn): ...  # Error: PYI050
def foo_never(arg: Never): ...



import abc

import pydantic


class Class:
    def badAllowed(this):
        pass

    def stillBad(this):
        pass

    if False:

        def badAllowed(this):
            pass

        def stillBad(this):
            pass

    @pydantic.validator
    def badAllowed(cls, my_field: str) -> str:
        pass

    @pydantic.validator
    def stillBad(cls, my_field: str) -> str:
        pass

    @pydantic.validator("my_field")
    def badAllowed(cls, my_field: str) -> str:
        pass

    @pydantic.validator("my_field")
    def stillBad(cls, my_field: str) -> str:
        pass

    @classmethod
    def badAllowed(cls):
        pass

    @classmethod
    def stillBad(cls):
        pass

    @abc.abstractclassmethod
    def badAllowed(cls):
        pass

    @abc.abstractclassmethod
    def stillBad(cls):
        pass


class PosOnlyClass:
    def badAllowed(this, blah, /, self, something: str):
        pass

    def stillBad(this, blah, /, self, something: str):
        pass



from abc import ABCMeta


class Class:
    def __init_subclass__(self, default_name, **kwargs):
        ...

    @classmethod
    def badAllowed(self, x, /, other):
        ...

    @classmethod
    def stillBad(self, x, /, other):
        ...


class MetaClass(ABCMeta):
    def badAllowed(self):
        pass

    def stillBad(self):
        pass



foo_tuple = (1, 2, 3)
foo_list = [1, 2, 3]
foo_set = {1, 2, 3}
foo_dict = {1: 2, 3: 4}
foo_int = 123

for i in list(foo_tuple):  # PERF101
    pass

for i in list(foo_list):  # PERF101
    pass

for i in list(foo_set):  # PERF101
    pass

for i in list((1, 2, 3)):  # PERF101
    pass

for i in list([1, 2, 3]):  # PERF101
    pass

for i in list({1, 2, 3}):  # PERF101
    pass

for i in list(
    {
    1,
    2,
    3,
    }
):
    pass

for i in list( # Comment
    {1, 2, 3}
):  # PERF101
    pass

for i in list(foo_dict):  # Ok
    pass

for i in list(1):  # Ok
    pass

for i in list(foo_int):  # Ok
    pass


import itertools

for i in itertools.product(foo_int):  # Ok
    pass



#!/usr/bin/env python3
"""Generate boilerplate for a new rule.

Example usage:

    python scripts/add_rule.py \
        --name PreferListBuiltin \
        --prefix PIE \
        --code 807 \
        --linter flake8-pie
"""
from __future__ import annotations

import argparse
import subprocess

from _utils import ROOT_DIR, dir_name, get_indent, pascal_case, snake_case


def main(*, name: str, prefix: str, code: str, linter: str) -> None:
    """Generate boilerplate for a new rule."""
    # Create a test fixture.
    filestem = f"{prefix}{code}" if linter != "pylint" else snake_case(name)
    with (
        ROOT_DIR
        / "crates/ruff_linter/resources/test/fixtures"
        / dir_name(linter)
        / f"{filestem}.py"
    ).open(
        "a",
    ):
        pass

    plugin_module = ROOT_DIR / "crates/ruff_linter/src/rules" / dir_name(linter)
    rule_name_snake = snake_case(name)

    # Add the relevant `#testcase` macro.
    mod_rs = plugin_module / "mod.rs"
    content = mod_rs.read_text()

    with mod_rs.open("w") as fp:
        has_added_testcase = False
        lines = []
        for line in content.splitlines():
            if not has_added_testcase and (
                line.strip() == "fn rules(rule_code: Rule, path: &Path) -> Result<()> {"
            ):
                indent = get_indent(line)
                lines.append(
                    f'{indent}#[test_case(Rule::{name}, Path::new("{filestem}.py"))]',
                )
                fp.write("\n".join(lines))
                fp.write("\n")
                lines.clear()
                has_added_testcase = True

            if has_added_testcase:
                fp.write(line)
                fp.write("\n")
            elif line.strip() == "":
                fp.write("\n".join(lines))
                fp.write("\n\n")
                lines.clear()
            else:
                lines.append(line)

    # Add the exports
    rules_dir = plugin_module / "rules"
    rules_mod = rules_dir / "mod.rs"

    contents = rules_mod.read_text()
    parts = contents.split("\n\n")

    new_pub_use = f"pub(crate) use {rule_name_snake}::*"
    new_mod = f"mod {rule_name_snake};"

    if len(parts) == 2:
        new_contents = parts[0]
        new_contents += "\n" + new_pub_use + ";"
        new_contents += "\n\n"
        new_contents += parts[1] + new_mod
        new_contents += "\n"

        rules_mod.write_text(new_contents)
    else:
        with rules_mod.open("a") as fp:
            fp.write(f"{new_pub_use};")
            fp.write("\n\n")
            fp.write(f"{new_mod}")
            fp.write("\n")

    # Add the relevant rule function.
    with (rules_dir / f"{rule_name_snake}.rs").open("w") as fp:
        fp.write(
            f"""\
use ruff_diagnostics::Violation;
use ruff_macros::{{derive_message_formats, violation}};

use crate::checkers::ast::Checker;

/// ## What it does
///
/// ## Why is this bad?
///
/// ## Example
/// ```python
/// ```
///
/// Use instead:
/// ```python
/// ```
#[violation]
pub struct {name};

impl Violation for {name} {{
    #[derive_message_formats]
    fn message(&self) -> String {{
        format!("TODO: write message: {{}}", todo!("implement message"))
    }}
}}
""",
        )
        fp.write(
            f"""
/// {prefix}{code}
pub(crate) fn {rule_name_snake}(checker: &mut Checker) {{}}
""",
        )

    text = ""
    with (ROOT_DIR / "crates/ruff_linter/src/codes.rs").open("r") as fp:
        while (line := next(fp)).strip() != f"// {linter}":
            text += line
        text += line

        lines = []
        while (line := next(fp)).strip() != "":
            lines.append(line)

        variant = pascal_case(linter)
        rule = f"""rules::{linter.split(" ")[0]}::rules::{name}"""
        lines.append(
            " " * 8
            + f"""({variant}, "{code}") => (RuleGroup::Unspecified, {rule}),\n""",
        )
        lines.sort()
        text += "".join(lines)
        text += "\n"
        text += fp.read()
    with (ROOT_DIR / "crates/ruff_linter/src/codes.rs").open("w") as fp:
        fp.write(text)

    _rustfmt(rules_mod)


def _rustfmt(path: str) -> None:
    subprocess.run(["rustfmt", path])


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Generate boilerplate for a new rule.",
        epilog=(
            "python scripts/add_rule.py "
            "--name PreferListBuiltin --code PIE807 --linter flake8-pie"
        ),
    )
    parser.add_argument(
        "--name",
        type=str,
        required=True,
        help=(
            "The name of the check to generate, in PascalCase "
            "(e.g., 'PreferListBuiltin')."
        ),
    )
    parser.add_argument(
        "--prefix",
        type=str,
        required=True,
        help="Prefix code for the plugin (e.g. 'PIE').",
    )
    parser.add_argument(
        "--code",
        type=str,
        required=True,
        help="The code of the check to generate (e.g., '807').",
    )
    parser.add_argument(
        "--linter",
        type=str,
        required=True,
        help="The source with which the check originated (e.g., 'flake8-pie').",
    )
    args = parser.parse_args()

    main(name=args.name, prefix=args.prefix, code=args.code, linter=args.linter)



import sys
from sys import version_info as python_version

if sys.version_info < (3, 9): ...  # OK

if sys.version_info >= (3, 9): ...  # OK

if sys.version_info == (3, 9): ...  # OK

if sys.version_info == (3, 9): ...  # Error: PYI006 Use only `<` and `>=` for version info comparisons

if sys.version_info <= (3, 10): ...  # Error: PYI006 Use only `<` and `>=` for version info comparisons

if sys.version_info <= (3, 10): ...  # Error: PYI006 Use only `<` and `>=` for version info comparisons

if sys.version_info > (3, 10): ...  # Error: PYI006 Use only `<` and `>=` for version info comparisons

if python_version > (3, 10): ...  # Error: PYI006 Use only `<` and `>=` for version info comparisons



import socket

from kombu import Connection, exceptions

try:
    conn = Connection(settings.CELERY_BROKER_URL)
    conn.ensure_connection(max_retries=2)
    conn._close()
except (socket.error, exceptions.OperationalError):
    return HttpResponseServerError("cache: cannot connect to broker.")



try:
    y = 6 + "7"
except TypeError:
    # RSE102
    raise ValueError()

try:
    x = 1 / 0
except ZeroDivisionError:
    raise

# RSE102
raise TypeError()

# RSE102
raise TypeError ()

# RSE102
raise TypeError \
    ()

# RSE102
raise TypeError \
    ();

# RSE102
raise TypeError(

)

# RSE102
raise (TypeError) (

)

# RSE102
raise TypeError(
    # Hello, world!
)

# OK
raise AssertionError

# OK
raise AttributeError("test message")


def return_error():
    return ValueError("Something")


# OK
raise return_error()


class Class:
    @staticmethod
    def error():
        return ValueError("Something")


# OK
raise Class.error()


import ctypes


# OK
raise ctypes.WinError(1)


# RSE102
raise IndexError()from ZeroDivisionError

raise IndexError()\
from ZeroDivisionError

raise IndexError() from ZeroDivisionError

raise IndexError();



# These remain unchanged
str(1)
str(*a)
str("foo", *a)
str(**k)
str("foo", **k)
str("foo", encoding="UTF-8")
str("foo"
    "bar")
str(b"foo")
bytes("foo", encoding="UTF-8")
bytes(*a)
bytes("foo", *a)
bytes("foo", **a)
bytes(b"foo"
      b"bar")
bytes("foo")
bytes(1)
f"{f'{str()}'}"
int(1.0)
int("1")
int(b"11")
int(10, base=2)
int("10", base=2)
int("10", 2)
float("1.0")
float(b"1.0")
bool(1)
bool(0)
bool("foo")
bool("")
bool(b"")
bool(1.0)
int().denominator

# These become string or byte literals
str()
str("foo")
str("""
foo""")
bytes()
bytes(b"foo")
bytes(b"""
foo""")
f"{str()}"
int()
int(1)
float()
float(1.0)
bool()
bool(True)
bool(False)

# These become a literal but retain parentheses
int(1).denominator



import sys

if sys.version_info < (3,0):
    print("py2")
else:
    print("py3")

if sys.version_info < (3,0):
    if True:
        print("py2!")
    else:
        print("???")
else:
    print("py3")

if sys.version_info < (3,0): print("PY2!")
else: print("PY3!")

if True:
    if sys.version_info < (3,0):
        print("PY2")
    else:
        print("PY3")

if sys.version_info < (3,0): print(1 if True else 3)
else:
    print("py3")

if sys.version_info < (3,0):
    def f():
        print("py2")
else:
    def f():
        print("py3")
        print("This the next")

if sys.version_info > (3,0):
    print("py3")
else:
    print("py2")


x = 1

if sys.version_info > (3,0):
    print("py3")
else:
    print("py2")
    # ohai

x = 1

if sys.version_info > (3,0): print("py3")
else: print("py2")

if sys.version_info > (3,):
    print("py3")
else:
    print("py2")

if True:
    if sys.version_info > (3,):
        print("py3")
    else:
        print("py2")

if sys.version_info < (3,):
    print("py2")
else:
    print("py3")

def f():
    if sys.version_info < (3,0):
        try:
            yield
        finally:
            pass
    else:
        yield


class C:
    def g():
        pass

    if sys.version_info < (3,0):
        def f(py2):
            pass
    else:
        def f(py3):
            pass

    def h():
        pass

if True:
    if sys.version_info < (3,0):
        2
    else:
        3

    # comment

if sys.version_info < (3,0):
    def f():
        print("py2")
    def g():
        print("py2")
else:
    def f():
        print("py3")
    def g():
        print("py3")

if True:
    if sys.version_info > (3,):
        print(3)
    # comment
    print(2+3)

if True:
    if sys.version_info > (3,): print(3)

if True:
    if sys.version_info > (3,):
        print(3)


if True:
    if sys.version_info <= (3, 0):
        expected_error = []
    else:
        expected_error = [
"<stdin>:1:5: Generator expression must be parenthesized",
"max(1 for i in range(10), key=lambda x: x+1)",
"    ^",
        ]


if sys.version_info <= (3, 0):
    expected_error = []
else:
    expected_error = [
"<stdin>:1:5: Generator expression must be parenthesized",
"max(1 for i in range(10), key=lambda x: x+1)",
"    ^",
    ]


if sys.version_info > (3,0):
    """this
is valid"""

    """the indentation on
    this line is significant"""

    "this is" \
    "allowed too"

    ("so is"
     "this for some reason")

if sys.version_info > (3, 0): expected_error = \
    []

if sys.version_info > (3, 0): expected_error = []

if sys.version_info > (3, 0): \
    expected_error = []

if True:
    if sys.version_info > (3, 0): expected_error = \
    []

if True:
    if sys.version_info > (3, 0): expected_error = []

if True:
    if sys.version_info > (3, 0): \
    expected_error = []

if sys.version_info < (3,12):
    print("py3")

if sys.version_info <= (3,12):
    print("py3")



import datetime

# qualified
datetime.datetime.utcfromtimestamp(1234)

from datetime import datetime

# unqualified
datetime.utcfromtimestamp(1234)

# uses `astimezone` method
datetime.utcfromtimestamp(1234).astimezone()



"""Generate an MkDocs-compatible `docs` and `mkdocs.yml` from the README.md."""
from __future__ import annotations

import argparse
import re
import shutil
import subprocess
from pathlib import Path
from typing import NamedTuple

import yaml


class Section(NamedTuple):
    """A section to include in the MkDocs documentation."""

    title: str
    filename: str
    generated: bool


SECTIONS: list[Section] = [
    Section("Overview", "index.md", generated=True),
    Section("Tutorial", "tutorial.md", generated=False),
    Section("Installation", "installation.md", generated=False),
    Section("Usage", "usage.md", generated=False),
    Section("Configuration", "configuration.md", generated=False),
    Section("Preview", "preview.md", generated=False),
    Section("Rules", "rules.md", generated=True),
    Section("Settings", "settings.md", generated=True),
    Section("Editor Integrations", "editor-integrations.md", generated=False),
    Section("FAQ", "faq.md", generated=False),
    Section("Contributing", "contributing.md", generated=True),
]


LINK_REWRITES: dict[str, str] = {
    "https://docs.astral.sh/ruff/": "index.md",
    "https://docs.astral.sh/ruff/configuration/": "configuration.md",
    "https://docs.astral.sh/ruff/configuration/#pyprojecttoml-discovery": (
        "configuration.md#pyprojecttoml-discovery"
    ),
    "https://docs.astral.sh/ruff/contributing/": "contributing.md",
    "https://docs.astral.sh/ruff/editor-integrations/": "editor-integrations.md",
    "https://docs.astral.sh/ruff/faq/#how-does-ruff-compare-to-flake8": (
        "faq.md#how-does-ruff-compare-to-flake8"
    ),
    "https://docs.astral.sh/ruff/installation/": "installation.md",
    "https://docs.astral.sh/ruff/rules/": "rules.md",
    "https://docs.astral.sh/ruff/rules/#error-e": "rules.md#error-e",
    "https://docs.astral.sh/ruff/settings/": "settings.md",
}


def clean_file_content(content: str, title: str) -> str:
    """Add missing title, fix the header depth, and remove trailing empty lines."""
    lines = content.splitlines()
    if lines[0].startswith("# "):
        return content

    in_code_block = False
    for i, line in enumerate(lines):
        if line.startswith("```"):
            in_code_block = not in_code_block
        if not in_code_block and line.startswith("#"):
            lines[i] = line[1:]

    # Remove trailing empty lines.
    for line in reversed(lines):
        if line == "":
            del lines[-1]
        else:
            break

    content = "\n".join(lines) + "\n"

    # Add a missing title.
    return f"# {title}\n\n" + content


def main() -> None:
    """Generate an MkDocs-compatible `docs` and `mkdocs.yml`."""
    subprocess.run(["cargo", "dev", "generate-docs"], check=True)

    with Path("README.md").open(encoding="utf8") as fp:
        content = fp.read()

    # Rewrite links to the documentation.
    for src, dst in LINK_REWRITES.items():
        before = content
        after = content.replace(f"({src})", f"({dst})")
        if before == after:
            msg = f"Unexpected link rewrite in README.md: {src}"
            raise ValueError(msg)
        content = after

    if m := re.search(r"\(https://docs.astral.sh/ruff/.*\)", content):
        msg = f"Unexpected absolute link to documentation: {m.group(0)}"
        raise ValueError(msg)

    Path("docs").mkdir(parents=True, exist_ok=True)

    # Split the README.md into sections.
    for title, filename, generated in SECTIONS:
        if not generated:
            continue

        with Path(f"docs/{filename}").open("w+") as f:
            if filename == "contributing.md":
                # Copy the CONTRIBUTING.md.
                shutil.copy("CONTRIBUTING.md", "docs/contributing.md")
                continue

            if filename == "settings.md":
                file_content = subprocess.check_output(
                    ["cargo", "dev", "generate-options"],
                    encoding="utf-8",
                )
            else:
                block = content.split(f"<!-- Begin section: {title} -->\n\n")
                if len(block) != 2:
                    msg = f"Section {title} not found in README.md"
                    raise ValueError(msg)

                block = block[1].split(f"\n<!-- End section: {title} -->")
                if len(block) != 2:
                    msg = f"Section {title} not found in README.md"
                    raise ValueError(msg)

                file_content = block[0]

                if filename == "rules.md":
                    file_content += "\n" + subprocess.check_output(
                        ["cargo", "dev", "generate-rules-table"],
                        encoding="utf-8",
                    )

            f.write(clean_file_content(file_content, title))

    # Add the nav section to mkdocs.yml.
    with Path("mkdocs.template.yml").open(encoding="utf8") as fp:
        config = yaml.safe_load(fp)
    config["nav"] = [{section.title: section.filename} for section in SECTIONS]

    with Path("mkdocs.generated.yml").open("w+") as fp:
        yaml.safe_dump(config, fp)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Generate an MkDocs-compatible `docs` and `mkdocs.yml`.",
    )
    args = parser.parse_args()
    main()



# Errors.
foo == "a" or foo == "b"

foo != "a" and foo != "b"

foo == "a" or foo == "b" or foo == "c"

foo != "a" and foo != "b" and foo != "c"

foo == a or foo == "b" or foo == 3  # Mixed types.

"a" == foo or "b" == foo or "c" == foo

"a" != foo and "b" != foo and "c" != foo

"a" == foo or foo == "b" or "c" == foo

foo == bar or baz == foo or qux == foo

foo == "a" or "b" == foo or foo == "c"

foo != "a" and "b" != foo and foo != "c"

foo == "a" or foo == "b" or "c" == bar or "d" == bar  # Multiple targets

foo.bar == "a" or foo.bar == "b"  # Attributes.

# OK
foo == "a" and foo == "b" and foo == "c"  # `and` mixed with `==`.

foo != "a" or foo != "b" or foo != "c"  # `or` mixed with `!=`.

foo == a or foo == b() or foo == c  # Call expression.

foo != a or foo() != b or foo != c  # Call expression.

foo in {"a", "b", "c"}  # Uses membership test already.

foo not in {"a", "b", "c"}  # Uses membership test already.

foo == "a"  # Single comparison.

foo != "a"  # Single comparison.

foo == "a" == "b" or foo == "c"  # Multiple comparisons.

foo == bar == "b" or foo == "c"  # Multiple comparisons.

foo == foo or foo == bar  # Self-comparison.

foo[0] == "a" or foo[0] == "b"  # Subscripts.

foo() == "a" or foo() == "b"  # Calls.



class Foo(BaseModel):
    name = StringField()
    # ....
    name = StringField()  # PIE794

    def remove(self) -> None:
        ...


class Foo(BaseModel):
    name: str = StringField()
    # ....
    name = StringField()  # PIE794

    def foo(self) -> None:
        ...


class User(BaseModel):
    bar: str = StringField()
    foo: bool = BooleanField()
    # ...
    bar = StringField()  # PIE794


class User(BaseModel):
    @property
    def buzz(self) -> str:
        ...

    @buzz.setter
    def buzz(self, value: str | int) -> None:
        ...


class User:
    bar: str = StringField()
    foo: bool = BooleanField()
    # ...
    bar = StringField()  # PIE794



"""This is @konstin's scripts for checking an entire checkout of ~2.1k packages for
panics, autofix errors and similar problems.

It's a less elaborate, more hacky version of check_ecosystem.py
"""
from __future__ import annotations

import json
import subprocess
import sys
from pathlib import Path
from subprocess import CalledProcessError
from typing import NamedTuple

from tqdm import tqdm


class Repository(NamedTuple):
    """A GitHub repository at a specific ref."""

    org: str
    repo: str
    ref: str | None


def main() -> None:
    ruff_args = sys.argv[1:]
    checkouts = Path("checkouts")
    out_dir = Path("ecosystem_all_results")
    github_search_json = Path("github_search.jsonl")
    # Somehow it doesn't like plain ruff
    ruff = Path.cwd().joinpath("ruff")

    out_dir.mkdir(parents=True, exist_ok=True)

    repositories = []
    for line in github_search_json.read_text().splitlines():
        item = json.loads(line)
        # Pick only the easier case for now.
        if item["path"] != "pyproject.toml":
            continue
        repositories.append(
            Repository(
                item["owner"],
                item["repo"],
                item.get("ref"),
            ),
        )

    successes = 0
    errors = 0
    for repository in tqdm(repositories):
        project_dir = checkouts.joinpath(f"{repository.org}:{repository.repo}")
        if not project_dir.is_dir():
            tqdm.write(f"Missing {project_dir}")
            errors += 1
            continue

        try:
            output = subprocess.run(
                [ruff, *ruff_args, "."],
                cwd=project_dir,
                capture_output=True,
                text=True,
            )
        except CalledProcessError as e:
            tqdm.write(f"Ruff failed on {project_dir}: {e}")
            errors += 1
            continue

        org_repo = f"{repository.org}:{repository.repo}"
        out_dir.joinpath(f"{org_repo}.stdout.txt").write_text(output.stdout)
        out_dir.joinpath(f"{org_repo}.stderr.txt").write_text(output.stderr)
        successes += 1
    print(f"Success: {successes} Error {errors}")


if __name__ == "__main__":
    main()



import mod.BadAllowed as badallowed
import mod.stillBad as stillbad

from mod import BadAllowed as badallowed
from mod import StillBad as stillbad

from mod import BadAllowed as bad_allowed
from mod import StillBad as still_bad



from django.db import models
from django.db.models import Model


# Models without __str__
class TestModel1(models.Model):
    new_field = models.CharField(max_length=10)

    class Meta:
        verbose_name = "test model"
        verbose_name_plural = "test models"

    @property
    def my_brand_new_property(self):
        return 1

    def my_beautiful_method(self):
        return 2


class TestModel2(Model):
    new_field = models.CharField(max_length=10)

    class Meta:
        verbose_name = "test model"
        verbose_name_plural = "test models"

    @property
    def my_brand_new_property(self):
        return 1

    def my_beautiful_method(self):
        return 2


class TestModel3(Model):
    new_field = models.CharField(max_length=10)

    class Meta:
        abstract = False

    @property
    def my_brand_new_property(self):
        return 1

    def my_beautiful_method(self):
        return 2


# Models with __str__
class TestModel4(Model):
    new_field = models.CharField(max_length=10)

    class Meta:
        verbose_name = "test model"
        verbose_name_plural = "test models"

    def __str__(self):
        return self.new_field

    @property
    def my_brand_new_property(self):
        return 1

    def my_beautiful_method(self):
        return 2


class TestModel5(models.Model):
    new_field = models.CharField(max_length=10)

    class Meta:
        verbose_name = "test model"
        verbose_name_plural = "test models"

    def __str__(self):
        return self.new_field

    @property
    def my_brand_new_property(self):
        return 1

    def my_beautiful_method(self):
        return 2


# Abstract models without str
class AbstractTestModel1(models.Model):
    new_field = models.CharField(max_length=10)

    class Meta:
        abstract = True

    @property
    def my_brand_new_property(self):
        return 1

    def my_beautiful_method(self):
        return 2


class AbstractTestModel2(Model):
    new_field = models.CharField(max_length=10)

    class Meta:
        abstract = True

    @property
    def my_brand_new_property(self):
        return 1

    def my_beautiful_method(self):
        return 2


# Abstract models with __str__


class AbstractTestModel3(Model):
    new_field = models.CharField(max_length=10)

    class Meta:
        abstract = True

    def __str__(self):
        return self.new_field

    @property
    def my_brand_new_property(self):
        return 1

    def my_beautiful_method(self):
        return 2


class AbstractTestModel4(models.Model):
    new_field = models.CharField(max_length=10)

    class Meta:
        abstract = True

    def __str__(self):
        return self.new_field

    @property
    def my_brand_new_property(self):
        return 1

    def my_beautiful_method(self):
        return 2


class AbstractTestModel5(models.Model):
    new_field = models.CharField(max_length=10)

    class Meta:
        abstract = False

    def __str__(self):
        return self.new_field

    @property
    def my_brand_new_property(self):
        return 1

    def my_beautiful_method(self):
        return 2



"""Test case expected to be run with `ignore_fully_untyped = True`."""


def ok_fully_untyped_1(a, b):
    pass


def ok_fully_untyped_2():
    pass


def ok_fully_typed_1(a: int, b: int) -> int:
    pass


def ok_fully_typed_2() -> int:
    pass


def ok_fully_typed_3(a: int, *args: str, **kwargs: str) -> int:
    pass


def error_partially_typed_1(a: int, b):
    pass


def error_partially_typed_2(a: int, b) -> int:
    pass


def error_partially_typed_3(a: int, b: int):
    pass


class X:
    def ok_untyped_method_with_arg(self, a):
        pass

    def ok_untyped_method(self):
        pass

    def error_typed_self(self: X):
        pass



__all__: list[str]  # Error: PYI035

__all__: list[str] = ["foo"]

class Foo:
    __all__: list[str]
    __match_args__: tuple[str, ...]  # Error: PYI035
    __slots__: tuple[str, ...]  # Error: PYI035

class Bar:
    __all__: list[str] = ["foo"]
    __match_args__: tuple[str, ...] = (1,)
    __slots__: tuple[str, ...] = "foo"



import os
import stat

keyfile = "foo"

os.chmod("/etc/passwd", 0o227)  # Error
os.chmod("/etc/passwd", 0o7)  # Error
os.chmod("/etc/passwd", 0o664)  # OK
os.chmod("/etc/passwd", 0o777)  # Error
os.chmod("/etc/passwd", 0o770)  # Error
os.chmod("/etc/passwd", 0o776)  # Error
os.chmod("/etc/passwd", 0o760)  # OK
os.chmod("~/.bashrc", 511)  # Error
os.chmod("/etc/hosts", 0o777)  # Error
os.chmod("/tmp/oh_hai", 0x1FF)  # Error
os.chmod("/etc/passwd", stat.S_IRWXU)  # OK
os.chmod(keyfile, 0o777)  # Error
os.chmod(keyfile, 0o7 | 0o70 | 0o700)  # Error
os.chmod(keyfile, stat.S_IRWXO | stat.S_IRWXG | stat.S_IRWXU)  # Error
os.chmod("~/hidden_exec", stat.S_IXGRP)  # Error
os.chmod("~/hidden_exec", stat.S_IXOTH)  # OK
os.chmod("/etc/passwd", stat.S_IWOTH)  # Error



def scope():
    # E731
    f = lambda x: 2 * x


def scope():
    # E731
    f = lambda x: 2 * x


def scope():
    # E731
    while False:
        this = lambda y, z: 2 * x


def scope():
    # E731
    f = lambda: (yield 1)


def scope():
    # E731
    f = lambda: (yield from g())


def scope():
    # OK
    f = object()
    f.method = lambda: "Method"


def scope():
    # OK
    f = {}
    f["a"] = lambda x: x**2


def scope():
    # OK
    f = []
    f.append(lambda x: x**2)


def scope():
    # OK
    f = g = lambda x: x**2


def scope():
    # OK
    lambda: "no-op"


class Scope:
    # E731
    f = lambda x: 2 * x


class Scope:
    from typing import Callable

    # E731
    f: Callable[[int], int] = lambda x: 2 * x


def scope():
    # E731
    from typing import Callable

    x: Callable[[int], int]
    if True:
        x = lambda: 1
    else:
        x = lambda: 2
    return x


def scope():
    # E731

    from typing import Callable, ParamSpec

    # ParamSpec cannot be used in this context, so do not preserve the annotation.
    P = ParamSpec("P")
    f: Callable[P, int] = lambda *args: len(args)


def scope():
    # E731

    from typing import Callable

    f: Callable[[], None] = lambda: None


def scope():
    # E731

    from typing import Callable

    f: Callable[..., None] = lambda a, b: None


def scope():
    # E731

    from typing import Callable

    f: Callable[[int], int] = lambda x: 2 * x


# Let's use the `Callable` type from `collections.abc` instead.
def scope():
    # E731

    from collections.abc import Callable

    f: Callable[[str, int], str] = lambda a, b: a * b


def scope():
    # E731

    from collections.abc import Callable

    f: Callable[[str, int], tuple[str, int]] = lambda a, b: (a, b)


def scope():
    # E731

    from collections.abc import Callable

    f: Callable[[str, int, list[str]], list[str]] = lambda a, b, /, c: [*c, a * b]


class TemperatureScales(Enum):
    CELSIUS = (lambda deg_c: deg_c)
    FAHRENHEIT = (lambda deg_c: deg_c * 9 / 5 + 32)


# Regression test for: https://github.com/astral-sh/ruff/issues/7141
def scope():
    # E731

    f = lambda: (
        i := 1,
    )



# TODO: comments starting with one of the configured task-tags sometimes are longer than line-length so that you can easily find them with `git grep`
# TODO comments starting with one of the configured task-tags sometimes are longer than line-length so that you can easily find them with `git grep`
#     TODO    comments starting with one of the configured task-tags sometimes are longer than line-length so that you can easily find them with `git grep`
# FIXME: comments starting with one of the configured task-tags sometimes are longer than line-length so that you can easily find them with `git grep`
# FIXME comments starting with one of the configured task-tags sometimes are longer than line-length so that you can easily find them with `git grep`
#     FIXME    comments starting with one of the configured task-tags sometimes are longer than line-length so that you can easily find them with `git grep`



import os
import subprocess

os.popen("chmod +w foo*")
subprocess.Popen("/bin/chown root: *", shell=True)
subprocess.Popen(["/usr/local/bin/rsync", "*", "some_where:"], shell=True)
subprocess.Popen("/usr/local/bin/rsync * no_injection_here:")
os.system("tar cf foo.tar bar/*")



# Errors
if a:
    b
elif c:
    b

if x == 1:
    for _ in range(20):
        print("hello")
elif x == 2:
    for _ in range(20):
        print("hello")

if x == 1:
    if True:
        for _ in range(20):
            print("hello")
elif x == 2:
    if True:
        for _ in range(20):
            print("hello")

if x == 1:
    if True:
        for _ in range(20):
            print("hello")
    elif False:
        for _ in range(20):
            print("hello")
elif x == 2:
    if True:
        for _ in range(20):
            print("hello")
    elif False:
        for _ in range(20):
            print("hello")

if (
    x == 1
    and y == 2
    and z == 3
    and a == 4
    and b == 5
    and c == 6
    and d == 7
    and e == 8
    and f == 9
    and g == 10
    and h == 11
    and i == 12
    and j == 13
    and k == 14
):
    pass
elif 1 == 2:
    pass

if result.eofs == "O":
    pass
elif result.eofs == "S":
    skipped = 1
elif result.eofs == "F":
    errors = 1
elif result.eofs == "E":
    errors = 1


# OK
def complicated_calc(*arg, **kwargs):
    return 42


def foo(p):
    if p == 2:
        return complicated_calc(microsecond=0)
    elif p == 3:
        return complicated_calc(microsecond=0, second=0)
    return None


a = False
b = True
c = True

if a:
    z = 1
elif b:
    z = 2
elif c:
    z = 1

# False negative (or arguably a different rule)
if result.eofs == "F":
    errors = 1
else:
    errors = 1

if a:
    # Ignore branches with diverging comments because it means we're repeating
    # the bodies because we have different reasons for each branch
    x = 1
elif c:
    x = 1


def foo():
    a = True
    b = False
    if a > b:  # end-of-line
        return 3
    elif a == b:
        return 3
    elif a < b:  # end-of-line
        return 4
    elif b is None:
        return 4



from os import chmod as xchmod, mkdir as xmkdir, sep as s
from os import makedirs as xmakedirs, rename as xrename, replace as xreplace
from os import rmdir as xrmdir, remove as xremove, unlink as xunlink
from os import getcwd as xgetcwd, readlink as xreadlink, stat as xstat
from os.path import abspath as xabspath, exists as xexists
from os.path import expanduser as xexpanduser, isdir as xisdir
from os.path import isfile as xisfile, islink as xislink, isabs as xisabs
from os.path import join as xjoin, basename as xbasename, dirname as xdirname
from os.path import samefile as xsamefile, splitext as xsplitext

p = "/foo"
q = "bar"

a = xabspath(p)
aa = xchmod(p)
aaa = xmkdir(p)
xmakedirs(p)
xrename(p)
xreplace(p)
xrmdir(p)
xremove(p)
xunlink(p)
xgetcwd(p)
b = xexists(p)
bb = xexpanduser(p)
bbb = xisdir(p)
bbbb = xisfile(p)
bbbbb = xislink(p)
xreadlink(p)
xstat(p)
xisabs(p)
xjoin(p, q)
s.join((p, q))
s.join([p, q])
xbasename(p)
xdirname(p)
xsamefile(p)
xsplitext(p)



# Errors
def main_function():
    try:
        process()
        handle()
        finish()
    except Exception as ex:
        logger.exception(f"Found an error: {ex}")  # TRY401


def main_function():
    try:
        process()
        handle()
        finish()
    except ValueError as bad:
        if True is False:
            for i in range(10):
                logger.exception(f"Found an error: {bad} {good}")  # TRY401
    except IndexError as bad:
        logger.exception(f"Found an error: {bad} {bad}")  # TRY401
    except Exception as bad:
        logger.exception(f"Found an error: {bad}")  # TRY401
        logger.exception(f"Found an error: {bad}")  # TRY401

        if True:
            logger.exception(f"Found an error: {bad}")  # TRY401


import logging

logger = logging.getLogger(__name__)


def func_fstr():
    try:
        ...
    except Exception as ex:
        logger.exception(f"Logging an error: {ex}")  # TRY401


def func_concat():
    try:
        ...
    except Exception as ex:
        logger.exception("Logging an error: " + str(ex))  # TRY401


def func_comma():
    try:
        ...
    except Exception as ex:
        logger.exception("Logging an error:", ex)  # TRY401


# OK
def main_function():
    try:
        process()
        handle()
        finish()
    except Exception as ex:
        logger.exception(f"Found an error: {er}")
        logger.exception(f"Found an error: {ex.field}")



# pylint: disable=missing-docstring,consider-using-f-string, pointless-statement

## Old style formatting

"%s %z" % ("hello", "world")  # [bad-format-character]

"%s" "%z" % ("hello", "world")  # [bad-format-character]

"""%s %z""" % ("hello", "world")  # [bad-format-character]

"""%s""" """%z""" % ("hello", "world")  # [bad-format-character]

## New style formatting

"{:s} {:y}".format("hello", "world")  # [bad-format-character]

"{:*^30s}".format("centered") # OK
"{:{s}}".format("hello", s="s")  # OK (nested placeholder value not checked)
"{:{s:y}}".format("hello", s="s")  # [bad-format-character] (nested placeholder format spec checked)
"{0:.{prec}g}".format(1.23, prec=15)  # OK (cannot validate after nested placeholder)
"{0:.{foo}{bar}{foobar}y}".format(...)  # OK (cannot validate after nested placeholders)
"{0:.{foo}x{bar}y{foobar}g}".format(...)  # OK (all nested placeholders are consumed without considering in between chars)

## f-strings

H, W = "hello", "world"
f"{H} {W}"
f"{H:s} {W:z}"  # [bad-format-character]

f"{1:z}"  # [bad-format-character]

## False negatives

print(("%" "z") % 1)



# SIM102
if a:
    if b:
        c

# SIM102
if a:
    if b:
        if c:
            d

# SIM102
if a:
    pass
elif b:
    if c:
        d

# SIM102
if a:
    # Unfixable due to placement of this comment.
    if b:
        c

# SIM102
if a:
    if b:
        # Fixable due to placement of this comment.
        c

# OK
if a:
    if b:
        c
    else:
        d

# OK
if __name__ == "__main__":
    if foo():
        ...

# OK
if a:
    d
    if b:
        c

while x > 0:
    # SIM102
    if y > 0:
        if z > 0:
            """this
is valid"""

            """the indentation on
            this line is significant"""

            "this is" \
"allowed too"

            ("so is"
"this for some reason")


# SIM102
if x > 0:
    if y > 0:
        """this
is valid"""

        """the indentation on
        this line is significant"""

        "this is" \
"allowed too"

        ("so is"
"this for some reason")

while x > 0:
    # SIM102
    if node.module:
        if node.module == "multiprocessing" or node.module.startswith(
            "multiprocessing."
        ):
            print("Bad module!")

# SIM102 (auto-fixable)
if node.module012345678:
    if node.module == "multiprocß9💣2ℝ" or node.module.startswith(
        "multiprocessing."
    ):
        print("Bad module!")

# SIM102 (not auto-fixable)
if node.module0123456789:
    if node.module == "multiprocß9💣2ℝ" or node.module.startswith(
        "multiprocessing."
    ):
        print("Bad module!")

# SIM102
# Regression test for https://github.com/apache/airflow/blob/145b16caaa43f0c42bffd97344df916c602cddde/airflow/configuration.py#L1161
if a:
    if b:
        if c:
            print("if")
elif d:
    print("elif")

# OK
if a:
    if b:
        print("foo")
else:
    print("bar")

# OK
if a:
    if b:
        if c:
            print("foo")
        else:
            print("bar")
else:
    print("bar")

# OK
if a:
    # SIM 102
    if b:
        if c:
            print("foo")
else:
    print("bar")


# OK
if a:
    if b:
        if c:
            print("foo")
        print("baz")
else:
    print("bar")


# OK
if False:
    if a:
        pass


# OK
if True:
    if a:
        pass


# SIM102
def f():
    if a:
        pass
    elif b:
        if c:
            d



"""Tests to determine first-party import classification.

For typing-only import detection tests, see `TCH002.py`.
"""


def f():
    import TYP001

    x: TYP001


def f():
    import TYP001

    print(TYP001)


def f():
    from . import TYP001

    x: TYP001


def f():
    from . import TYP001

    print(TYP001)



import typing
from typing import Literal, TypeAlias, Union

A: str | Literal["foo"]
B: TypeAlias = typing.Union[Literal[b"bar", b"foo"], bytes, str]
C: TypeAlias = typing.Union[Literal[5], int, typing.Union[Literal["foo"], str]]
D: TypeAlias = typing.Union[Literal[b"str_bytes", 42], bytes, int]

def func(x: complex | Literal[1J], y: Union[Literal[3.14], float]): ...

# OK
A: Literal["foo"]
B: TypeAlias = Literal[b"bar", b"foo"]
C: TypeAlias = typing.Union[Literal[5], Literal["foo"]]
D: TypeAlias = Literal[b"str_bytes", 42]

def func(x: Literal[1J], y: Literal[3.14]): ...



"""
Test for else-if-used
"""


def ok0():
    """Should not trigger on elif"""
    if 1:
        pass
    elif 2:
        pass


def ok1():
    """If the orelse has more than 1 item in it, shouldn't trigger"""
    if 1:
        pass
    else:
        print()
        if 1:
            pass


def ok2():
    """If the orelse has more than 1 item in it, shouldn't trigger"""
    if 1:
        pass
    else:
        if 1:
            pass
        print()


def not_ok0():
    if 1:
        pass
    else:
        if 2:
            pass


def not_ok1():
    if 1:
        pass
    else:
        if 2:
            pass
        else:
            pass


# Regression test for https://github.com/apache/airflow/blob/f1e1cdcc3b2826e68ba133f350300b5065bbca33/airflow/models/dag.py#L1737
def not_ok2():
    if True:
        print(1)
    elif True:
        print(2)
    else:
        if True:
            print(3)
        else:
            print(4)




#: W605:1:10
regex = '\.png$'

#: W605:2:1
regex = '''
\.png$
'''

#: W605:2:6
f(
    '\_'
)

#: W605:4:6
"""
multi-line
literal
with \_ somewhere
in the middle
"""

#: W605:1:38
value = 'new line\nand invalid escape \_ here'


def f():
    #: W605:1:11
    return'\.png$'

#: Okay
regex = r'\.png$'
regex = '\\.png$'
regex = r'''
\.png$
'''
regex = r'''
\\.png$
'''
s = '\\'
regex = '\w'  # noqa
regex = '''
\w
'''  # noqa

regex = '\\\_'



import typing
import typing_extensions
from typing import Literal

# Shouldn't affect non-union field types.
field1: Literal[1]  # OK

# Should emit for duplicate field types.
field2: Literal[1] | Literal[2]  # Error

# Should emit for union types in arguments.
def func1(arg1: Literal[1] | Literal[2]):  # Error
    print(arg1)


# Should emit for unions in return types.
def func2() -> Literal[1] | Literal[2]:  # Error
    return "my Literal[1]ing"


# Should emit in longer unions, even if not directly adjacent.
field3: Literal[1] | Literal[2] | str  # Error
field4: str | Literal[1] | Literal[2]  # Error
field5: Literal[1] | str | Literal[2]  # Error
field6: Literal[1] | bool | Literal[2] | str  # Error

# Should emit for non-type unions.
field7 = Literal[1] | Literal[2]  # Error

# Should emit for parenthesized unions.
field8: Literal[1] | (Literal[2] | str)  # Error

# Should handle user parentheses when fixing.
field9: Literal[1] | (Literal[2] | str)  # Error
field10: (Literal[1] | str) | Literal[2]  # Error

# Should emit for union in generic parent type.
field11: dict[Literal[1] | Literal[2], str]  # Error

# Should emit for unions with more than two cases
field12: Literal[1] | Literal[2] | Literal[3]  # Error
field13: Literal[1] | Literal[2] | Literal[3] | Literal[4]  # Error

# Should emit for unions with more than two cases, even if not directly adjacent
field14: Literal[1] | Literal[2] | str | Literal[3]  # Error

# Should emit for unions with mixed literal internal types
field15: Literal[1] | Literal["foo"] | Literal[True]  # Error

# Shouldn't emit for duplicate field types with same value; covered by Y016
field16: Literal[1] | Literal[1]  # OK

# Shouldn't emit if in new parent type
field17: Literal[1] | dict[Literal[2], str]  # OK

# Shouldn't emit if not in a union parent
field18: dict[Literal[1], Literal[2]]  # OK

# Should respect name of literal type used
field19: typing.Literal[1] | typing.Literal[2]  # Error

# Should emit in cases with newlines
field20: typing.Union[
    Literal[
        1  # test
    ],
    Literal[2],
]  # Error, newline and comment will not be emitted in message

# Should handle multiple unions with multiple members
field21: Literal[1, 2] | Literal[3, 4]  # Error

# Should emit in cases with `typing.Union` instead of `|`
field22: typing.Union[Literal[1], Literal[2]]  # Error

# Should emit in cases with `typing_extensions.Literal`
field23: typing_extensions.Literal[1] | typing_extensions.Literal[2]  # Error

# Should emit in cases with nested `typing.Union`
field24: typing.Union[Literal[1], typing.Union[Literal[2], str]]  # Error

# Should emit in cases with mixed `typing.Union` and `|`
field25: typing.Union[Literal[1], Literal[2] | str]  # Error

# Should emit only once in cases with multiple nested `typing.Union`
field24: typing.Union[Literal[1], typing.Union[Literal[2], typing.Union[Literal[3], Literal[4]]]]  # Error



input = [1, 2, 3]
otherInput = [2, 3, 4]
foo = [1, 2, 3, 4]

# OK
zip(input, otherInput)  # different inputs
zip(input, otherInput[1:])  # different inputs
zip(input, input[2:])  # not successive
zip(input[:-1], input[2:])  # not successive
list(zip(input, otherInput))  # nested call
zip(input, input[1::2])  # not successive
zip(foo[:-1], foo[1:], foo, strict=False)  # more than 2 inputs
zip(foo[:-1], foo[1:], foo, strict=True)  # more than 2 inputs

# Errors
zip(input, input[1:])
zip(input, input[1::1])
zip(input[:-1], input[1:])
zip(input[1:], input[2:])
zip(input[1:-1], input[2:])
list(zip(input, input[1:]))
list(zip(input[:-1], input[1:]))
zip(foo[:-1], foo[1:], strict=True)
zip(foo[:-1], foo[1:], strict=False)
zip(foo[:-1], foo[1:], strict=bool(foo))



data = ["some", "Data"]

# Ok
{value: value.upper() for value in data}
{value.lower(): value.upper() for value in data}
{v: v*v for v in range(10)}
{(0, "a", v): v*v for v in range(10)} # Tuple with variable

# Errors
{"key": value.upper() for value in data}
{True: value.upper() for value in data}
{0: value.upper() for value in data}
{(1, "a"): value.upper() for value in data} # constant tuple



import typing
import typing_extensions
from typing import Literal

# Shouldn't affect non-union field types.
field1: Literal[1]  # OK

# Should emit for duplicate field types.
field2: Literal[1] | Literal[2]  # Error

# Should emit for union types in arguments.
def func1(arg1: Literal[1] | Literal[2]):  # Error
    print(arg1)


# Should emit for unions in return types.
def func2() -> Literal[1] | Literal[2]:  # Error
    return "my Literal[1]ing"


# Should emit in longer unions, even if not directly adjacent.
field3: Literal[1] | Literal[2] | str  # Error
field4: str | Literal[1] | Literal[2]  # Error
field5: Literal[1] | str | Literal[2]  # Error
field6: Literal[1] | bool | Literal[2] | str  # Error

# Should emit for non-type unions.
field7 = Literal[1] | Literal[2]  # Error

# Should emit for parenthesized unions.
field8: Literal[1] | (Literal[2] | str)  # Error

# Should handle user parentheses when fixing.
field9: Literal[1] | (Literal[2] | str)  # Error
field10: (Literal[1] | str) | Literal[2]  # Error

# Should emit for union in generic parent type.
field11: dict[Literal[1] | Literal[2], str]  # Error



from itertools import count, cycle, repeat

# Errors
zip()
zip(range(3))
zip("a", "b")
zip("a", "b", *zip("c"))
zip(zip("a"), strict=False)
zip(zip("a", strict=True))

# OK
zip(range(3), strict=True)
zip("a", "b", strict=False)
zip("a", "b", "c", strict=True)

# OK (infinite iterators).
zip([1, 2, 3], cycle("ABCDEF"))
zip([1, 2, 3], count())
zip([1, 2, 3], repeat(1))
zip([1, 2, 3], repeat(1, None))
zip([1, 2, 3], repeat(1, times=None))

# Errors (limited iterators).
zip([1, 2, 3], repeat(1, 1))
zip([1, 2, 3], repeat(1, times=4))



import sys
import typing
from typing import TypedDict


class _UnusedTypedDict(TypedDict):
    foo: str


class _UnusedTypedDict2(typing.TypedDict):
    bar: int


# OK
class _UsedTypedDict(TypedDict):
   foo: bytes


class _CustomClass(_UsedTypedDict):
   bar: list[int]


if sys.version_info >= (3, 10):
   class _UsedTypedDict2(TypedDict):
       foo: int
else:
   class _UsedTypedDict2(TypedDict):
       foo: float


class _CustomClass2(_UsedTypedDict2):
   bar: list[int]



# UP035
from collections import Mapping

from collections import Mapping as MAP

from collections import Mapping, Sequence

from collections import Counter, Mapping

from collections import (Counter, Mapping)

from collections import (Counter,
                         Mapping)

from collections import Counter, \
                         Mapping

from collections import Counter, Mapping, Sequence

from collections import Mapping as mapping, Counter

if True:
    from collections import Mapping, Counter

if True:
    if True:
        pass
    from collections import Mapping, Counter

if True: from collections import Mapping

import os
from collections import Counter, Mapping
import sys

if True:
    from collections import (
        Mapping,
        Callable,
        Bad,
        Good,
    )

from typing import Callable, Match, Pattern, List, OrderedDict, AbstractSet, ContextManager

if True: from collections import (
    Mapping, Counter)

# Bad imports from PYI027 that are now handled by PYI022 (UP035)
from typing import ContextManager
from typing import OrderedDict
from typing_extensions import OrderedDict
from typing import Callable
from typing import ByteString
from typing import Container
from typing import Hashable
from typing import ItemsView
from typing import Iterable
from typing import Iterator
from typing import KeysView
from typing import Mapping
from typing import MappingView
from typing import MutableMapping
from typing import MutableSequence
from typing import MutableSet
from typing import Sequence
from typing import Sized
from typing import ValuesView
from typing import Awaitable
from typing import AsyncIterator
from typing import AsyncIterable
from typing import Coroutine
from typing import Collection
from typing import AsyncGenerator
from typing import Reversible
from typing import Generator
from typing import Callable
from typing import cast

# OK
from a import b

# Ok: `typing_extensions` contains backported improvements.
from typing_extensions import SupportsIndex

# Ok: `typing_extensions` contains backported improvements.
from typing_extensions import NamedTuple

# Ok: `typing_extensions` supports `frozen_default` (backported from 3.12).
from typing_extensions import dataclass_transform



foo(**{"bar": True})  # PIE804

foo(**{"r2d2": True})  # PIE804

Foo.objects.create(**{"bar": True})  # PIE804

Foo.objects.create(**{"_id": some_id})  # PIE804

Foo.objects.create(**{**bar})  # PIE804


foo(**{**data, "foo": "buzz"})
foo(**buzz)
foo(**{"bar-foo": True})
foo(**{"bar foo": True})
foo(**{"1foo": True})
foo(**{buzz: True})
foo(**{"": True})
foo(**{f"buzz__{bar}": True})
abc(**{"for": 3})



"""Test: module bindings are preferred over local bindings, for deferred annotations."""

from __future__ import annotations

from typing import TypeAlias, List


class Class:
    List: TypeAlias = List

    def bar(self) -> List:
        pass



def bar(): ...  # OK
def bar():
    pass  # OK

def bar():
    """oof"""  # OK

def oof():  # ERROR PYI048
    """oof"""
    print("foo")

def foo():  # ERROR PYI048
    """foo"""
    print("foo")
    print("foo")

def buzz():  # ERROR PYI048
    print("fizz")
    print("buzz")
    print("test")



# Below is black stable style
# In preview style, black always breaks the right side first

if True:
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa[
        bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
    ] = cccccccc.ccccccccccccc.cccccccc

    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa[
        bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
    ] = cccccccc.ccccccccccccc().cccccccc

    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa[
        bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb
    ] = cccccccc.ccccccccccccc(d).cccccccc

    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa[bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb] = (
        cccccccc.ccccccccccccc(d).cccccccc + e
    )

    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa[bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb] = (
        cccccccc.ccccccccccccc.cccccccc + e
    )
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa[bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb] = (
        cccccccc.ccccccccccccc.cccccccc
        + eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee
    )

    self._cache: dict[
        DependencyCacheKey, list[list[DependencyPackage]]
    ] = collections.defaultdict(list)
    self._cached_dependencies_by_level: dict[
        int, list[DependencyCacheKey]
    ] = collections.defaultdict(list)



import subprocess

# Errors.
subprocess.run("ls")
subprocess.run("ls", shell=True)

# Non-errors.
subprocess.run("ls", check=True)
subprocess.run("ls", check=False)
subprocess.run("ls", shell=True, check=True)
subprocess.run("ls", shell=True, check=False)
foo.run("ls")  # Not a subprocess.run call.
subprocess.bar("ls")  # Not a subprocess.run call.



# Errors
class A():
    pass


class A() \
    :
    pass


class A \
        ():
    pass


@decorator()
class A():
    pass

@decorator
class A():
    pass

# OK
class A:
    pass


class A(A):
    pass


class A(metaclass=type):
    pass


@decorator()
class A:
    pass

@decorator
class A:
    pass



while True:
    try:
        pass
    finally:
        continue  # [continue-in-finally]

while True:
    try:
        pass
    except Exception:
        continue
    finally:
        try:
            pass
        finally:
            continue  # [continue-in-finally]
        pass

while True:
    try:
        pass
    finally:
        test = "aa"
        match test:
            case "aa":
                continue  # [continue-in-finally]

while True:
    try:
        pass
    finally:
        with "aa" as f:
            continue  # [continue-in-finally]

while True:
    try:
        pass
    finally:
        if True:
            continue  # [continue-in-finally]
        continue  # [continue-in-finally]

        def test():
            while True:
                continue
            try:
                pass
            finally:
                continue  # [continue-in-finally]


while True:
    try:
        pass
    finally:
        continue  # [continue-in-finally]

        def test():
            while True:
                continue


while True:
    try:
        pass
    finally:
        for i in range(12):
            continue
        continue  # [continue-in-finally]

        while True:
            pass
        else:
            continue  # [continue-in-finally]

        def test():
            continue
            while True:
                continue


while True:
    try:
        pass
    finally:
        if True:
            pass
        elif False:
            continue  # [continue-in-finally]
        else:
            continue  # [continue-in-finally]
            for i in range(10):
                pass
            else:
                continue  # [continue-in-finally]



# Should not emit for valid access with index
var = "abc"[0]
var = f"abc"[0]
var = [1, 2, 3][0]
var = (1, 2, 3)[0]
var = b"abc"[0]

# Should not emit for valid access with slice
var = "abc"[0:2]
var = f"abc"[0:2]
var = b"abc"[0:2]
var = [1, 2, 3][0:2]
var = (1, 2, 3)[0:2]
var = [1, 2, 3][None:2]
var = [1, 2, 3][0:None]
var = [1, 2, 3][:2]
var = [1, 2, 3][0:]

# Should emit for invalid access on strings
var = "abc"["x"]
var = f"abc"["x"]

# Should emit for invalid access on bytes
var = b"abc"["x"]

# Should emit for invalid access on lists and tuples
var = [1, 2, 3]["x"]
var = (1, 2, 3)["x"]

# Should emit for invalid access on list comprehensions
var = [x for x in range(10)]["x"]

# Should emit for invalid access using tuple
var = "abc"[1, 2]

# Should emit for invalid access using string
var = [1, 2]["x"]

# Should emit for invalid access using float
var = [1, 2][0.25]

# Should emit for invalid access using dict
var = [1, 2][{"x": "y"}]

# Should emit for invalid access using dict comp
var = [1, 2][{x: "y" for x in range(2)}]

# Should emit for invalid access using list 
var = [1, 2][2, 3]

# Should emit for invalid access using list comp
var = [1, 2][[x for x in range(2)]]

# Should emit on invalid access using set
var = [1, 2][{"x", "y"}]

# Should emit on invalid access using set comp
var = [1, 2][{x for x in range(2)}]

# Should emit on invalid access using bytes
var = [1, 2][b"x"]

# Should emit for non-integer slice start
var = [1, 2, 3]["x":2]
var = [1, 2, 3][f"x":2]
var = [1, 2, 3][1.2:2]
var = [1, 2, 3][{"x"}:2]
var = [1, 2, 3][{x for x in range(2)}:2]
var = [1, 2, 3][{"x": x for x in range(2)}:2]
var = [1, 2, 3][[x for x in range(2)]:2]

# Should emit for non-integer slice end
var = [1, 2, 3][0:"x"]
var = [1, 2, 3][0:f"x"]
var = [1, 2, 3][0:1.2]
var = [1, 2, 3][0:{"x"}]
var = [1, 2, 3][0:{x for x in range(2)}]
var = [1, 2, 3][0:{"x": x for x in range(2)}]
var = [1, 2, 3][0:[x for x in range(2)]]

# Should emit for non-integer slice step
var = [1, 2, 3][0:1:"x"]
var = [1, 2, 3][0:1:f"x"]
var = [1, 2, 3][0:1:1.2]
var = [1, 2, 3][0:1:{"x"}]
var = [1, 2, 3][0:1:{x for x in range(2)}]
var = [1, 2, 3][0:1:{"x": x for x in range(2)}]
var = [1, 2, 3][0:1:[x for x in range(2)]]

# Should emit for non-integer slice start and end; should emit twice with specific ranges
var = [1, 2, 3]["x":"y"]

# Should emit once for repeated invalid access
var = [1, 2, 3]["x"]["y"]["z"]

# Cannot emit on invalid access using variable in index
x = "x"
var = "abc"[x]

# Cannot emit on invalid access using call
def func():
    return 1
var = "abc"[func()]

# Cannot emit on invalid access using a variable in parent
x = [1, 2, 3]
var = x["y"]

# Cannot emit for invalid access on byte array
var = bytearray(b"abc")["x"]

# Cannot emit for slice bound using variable
x = "x"
var = [1, 2, 3][0:x]
var = [1, 2, 3][x:1]



import collections.abc
import typing
from collections.abc import Iterator, Iterable

class NoReturn:
    def __iter__(self): ...

class TypingIterableTReturn:
    def __iter__(self) -> typing.Iterable[int]: ...  # Error: PYI045
    def not_iter(self) -> typing.Iterable[int]: ...

class TypingIterableReturn:
    def __iter__(self) -> typing.Iterable: ...  # Error: PYI045
    def not_iter(self) -> typing.Iterable: ...

class CollectionsIterableTReturn:
    def __iter__(self) -> collections.abc.Iterable[int]: ...  # Error: PYI045
    def not_iter(self) -> collections.abc.Iterable[int]: ...

class CollectionsIterableReturn:
    def __iter__(self) -> collections.abc.Iterable: ...  # Error: PYI045
    def not_iter(self) -> collections.abc.Iterable: ...

class IterableReturn:
    def __iter__(self) -> Iterable: ...  # Error: PYI045

class IteratorReturn:
    def __iter__(self) -> Iterator: ...

class IteratorTReturn:
    def __iter__(self) -> Iterator[int]: ...

class TypingIteratorReturn:
    def __iter__(self) -> typing.Iterator: ...

class TypingIteratorTReturn:
    def __iter__(self) -> typing.Iterator[int]: ...

class CollectionsIteratorReturn:
    def __iter__(self) -> collections.abc.Iterator: ...

class CollectionsIteratorTReturn:
    def __iter__(self) -> collections.abc.Iterator[int]: ...

class TypingAsyncIterableTReturn:
    def __aiter__(self) -> typing.AsyncIterable[int]: ...  # Error: PYI045

class TypingAsyncIterableReturn:
    def __aiter__(self) -> typing.AsyncIterable: ...  # Error: PYI045



"""Test: module bindings are preferred over local bindings, for deferred annotations."""

from __future__ import annotations

import datetime
from typing import Optional


class Class:
    datetime: Optional[datetime.datetime]



import urllib.request
import requests
import httpx


async def foo():
    urllib.request.urlopen("http://example.com/foo/bar").read()


async def foo():
    requests.get()


async def foo():
    httpx.get()


async def foo():
    requests.post()


async def foo():
    httpx.post()



import secrets
from random import random, choice

a = "Hello"
ok1 = " ".join([a, " World"])  # OK
ok2 = "".join(["Finally, ", a, " World"])  # OK
ok3 = "x".join(("1", "2", "3"))  # OK
ok4 = "y".join([1, 2, 3])  # Technically OK, though would've been an error originally
ok5 = "a".join([random(), random()])  # OK (simple calls)
ok6 = "a".join([secrets.token_urlsafe(), secrets.token_hex()])  # OK (attr calls)

nok1 = "x".join({"4", "5", "yee"})  # Not OK (set)
nok2 = a.join(["1", "2", "3"])  # Not OK (not a static joiner)
nok3 = "a".join(a)  # Not OK (not a static joinee)
nok4 = "a".join([a, a, *a])  # Not OK (not a static length)
nok5 = "a".join([choice("flarp")])  # Not OK (not a simple call)
nok6 = "a".join(x for x in "feefoofum")  # Not OK (generator)
nok7 = "a".join([f"foo{8}", "bar"])  # Not OK (contains an f-string)


# Regression test for: https://github.com/astral-sh/ruff/issues/7197
def create_file_public_url(url, filename):
    return''.join([url, filename])



import os.path
from pathlib import Path
from os.path import getatime

os.path.getatime("filename")
os.path.getatime(b"filename")
os.path.getatime(Path("filename"))


getatime("filename")
getatime(b"filename")
getatime(Path("filename"))



import pytest


@pytest.mark.foo(scope="module")
def ok_with_parameters_regardless_of_config():
    pass


# Without parentheses


@pytest.mark.foo
def test_something():
    pass


@pytest.mark.foo
class TestClass:
    def test_something():
        pass


class TestClass:
    @pytest.mark.foo
    def test_something():
        pass


class TestClass:
    @pytest.mark.foo
    class TestNestedClass:
        def test_something():
            pass


class TestClass:
    class TestNestedClass:
        @pytest.mark.foo
        def test_something():
            pass


# With parentheses


@pytest.mark.foo()
def test_something():
    pass


@pytest.mark.foo()
class TestClass:
    def test_something():
        pass


class TestClass:
    @pytest.mark.foo()
    def test_something():
        pass


class TestClass:
    @pytest.mark.foo()
    class TestNestedClass:
        def test_something():
            pass


class TestClass:
    class TestNestedClass:
        @pytest.mark.foo()
        def test_something():
            pass



# T007 - accepted
# TODO(evanrittenhouse): this has a space after a colon
# TODO: so does this
# T007 - errors
# TODO(evanrittenhouse):this has no space after a colon
# TODO (evanrittenhouse):this doesn't either
# TODO:neither does this
# FIXME:and lastly neither does this
# foo # TODO:this is really the last one
# TODO this colon doesn't terminate the tag, so don't check it. https://www.google.com



import os

# Check all functions.
subprocess.Popen("true")
subprocess.call("true")
subprocess.check_call("true")
subprocess.check_output("true")
subprocess.run("true")
os.system("true")
os.popen("true")
os.popen2("true")
os.popen3("true")
os.popen4("true")
popen2.popen2("true")
popen2.popen3("true")
popen2.popen4("true")
popen2.Popen3("true")
popen2.Popen4("true")
commands.getoutput("true")
commands.getstatusoutput("true")
os.execl("true")
os.execle("true")
os.execlp("true")
os.execlpe("true")
os.execv("true")
os.execve("true")
os.execvp("true")
os.execvpe("true")
os.spawnl("true")
os.spawnle("true")
os.spawnlp("true")
os.spawnlpe("true")
os.spawnv("true")
os.spawnve("true")
os.spawnvp("true")
os.spawnvpe("true")
os.startfile("true")

# Check it does not fail for full paths.
os.system("/bin/ls")
os.system("./bin/ls")
os.system(["/bin/ls"])
os.system(["/bin/ls", "/tmp"])
os.system(r"C:\\bin\ls")



def not_checked():
    import math


def unconventional():
    import altair
    import matplotlib.pyplot
    import numpy
    import pandas
    import seaborn
    import tkinter
    import networkx


def unconventional_aliases():
    import altair as altr
    import matplotlib.pyplot as plot
    import numpy as nmp
    import pandas as pdas
    import seaborn as sbrn
    import tkinter as tkr
    import networkx as nxy

def conventional_aliases():
    import altair as alt
    import matplotlib.pyplot as plt
    import numpy as np
    import pandas as pd
    import seaborn as sns
    import tkinter as tk
    import networkx as nx



# T002 - accepted
# TODO (evanrittenhouse): this has an author
# TODO(evanrittenhouse): this has an author
# TODO (evanrittenhouse) and more: this has an author
# TODO(evanrittenhouse) and more: this has an author
# TODO@mayrholu: this has an author
# TODO @mayrholu: this has an author
# TODO@mayrholu and more: this has an author
# TODO @mayrholu and more: this has an author
# T002 - errors
# TODO: this has no author
# FIXME: neither does this
# TODO : and neither does this
# foo # TODO: this doesn't either



# Violations of PYI012


class OneAttributeClass:
    value: int
    pass  # PYI012 Class body must not contain `pass`


class OneAttributeClassRev:
    pass  # PYI012 Class body must not contain `pass`
    value: int


class DocstringClass:
    """
    My body only contains pass.
    """

    pass  # PYI012 Class body must not contain `pass`


class NonEmptyChild(Exception):
    value: int
    pass  # PYI012 Class body must not contain `pass`


class NonEmptyChild2(Exception):
    pass  # PYI012 Class body must not contain `pass`
    value: int


class NonEmptyWithInit:
    value: int
    pass  # PYI012 Class body must not contain `pass`

    def __init__():
        pass


# Not violations (of PYI012)


class EmptyClass:
    pass  # Y009 Empty body should contain `...`, not `pass`


class EmptyOneLine:
    pass  # Y009 Empty body should contain `...`, not `pass`


class Dog:
    eyes: int = 2


class EmptyEllipsis:
    ...


class NonEmptyEllipsis:
    value: int
    ...  # Y013 Non-empty class body must not contain `...`


class WithInit:
    value: int = 0

    def __init__():
        pass


def function():
    pass


pass
